from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "https://supremecourt.nebraska.gov/courts/supreme-court/opinions"
        )
        self.status = "Published"

    def _process_html(self):
        path = {
            "group": '//div[contains(@class, "view-grouping")]',
            "date": './/span[contains(@class, "date-display-single")]',
            "row": ".//tbody/tr",
        }
        for group in self.html.xpath(path["group"]):
            date_element = group.xpath(path["date"])
            if not date_element:
                continue
            date = date_element[0].text_content()
            for row in group.xpath(path["row"]):
                url = row.xpath(".//td[3]//a/@href")

                # skip rows without url in third cell
                if not url:
                    continue

                self.cases.append(
                    {
                        "date": date,
                        "docket": row.xpath(".//td[1]")[0].text_content(),
                        "name": row.xpath(".//td[3]")[0].text_content(),
                        "neutral_citation": row.xpath(".//td[2]")[
                            0
                        ].text_content(),
                        "url": url[0],
                    }
                )



from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www.courtswv.gov/supreme-court/opinions.html"
        self.court_id = self.__module__
        self.cell_path = "//table/tbody/tr/td[%d]"

    def _get_case_names(self):
        return [
            anchor.text_content()
            for anchor in self.html.xpath("%s/a[1]" % (self.cell_path % 3))
        ]

    def _get_download_urls(self):
        return [
            href
            for href in self.html.xpath("%s/a[1]/@href" % (self.cell_path % 3))
        ]

    def _get_case_dates(self):
        return [
            convert_date_string(cell.text_content().strip())
            for cell in self.html.xpath(self.cell_path % 1)
        ]

    def _get_docket_numbers(self):
        return [
            cell.text_content().lower().strip()
            for cell in self.html.xpath(self.cell_path % 2)
        ]

    def _get_precedential_statuses(self):
        codes = {
            "MD": "Published",
            "SO": "Published",
            "PC": "Published",
            "SEP": "Separate",
        }
        return self.decode_cell_text(self.cell_path % 5, codes)

    def _get_nature_of_suit(self):
        # List is sourced from JS in scraped HTML
        codes = {
            "CR-F": "Felony (non-Death Penalty)",
            "CR-M": "Misdemeanor",
            "CR-O": "Criminal-Other",
            "TCR": "Tort, Contract, and Real Property",
            "PR": "Probate",
            "FAM": "Family",
            "JUV": "Juvenile",
            "CIV-O": "Civil-Other",
            "WC": "Workers Compensation",
            "TAX": "Revenue (Tax)",
            "ADM": "Administrative Agency-Other",
            "MISC": "Appeal by Right-Other",
            "OJ-H": "Habeas Corpus",
            "OJ-M": "Writ Application-Other",
            "OJ-P": "Writ Application-Other",
            "L-ADM": "Bar Admis   sion",
            "L-DISC": "Bar Discipline/Eligibility",
            "L-DISC-O": "Bar/Judiciary Proceeding-Other",
            "J-DISC": "Bar/Judiciary Proceeding-Other",
            "CERQ": "Certified Question",
            "OJ-O": "Original Proceeding/Appellate Matter-Other",
            "POST": "Post-Conviction Appeal",
        }
        return self.decode_cell_text(self.cell_path % 4, codes)

    def decode_cell_text(self, path, codes):
        results = []
        for cell in self.html.xpath(path):
            code = cell.text_content().strip()
            results.append(codes[code] if code in codes else "Unknown")
        return results



# Author: Michael Lissner
# Date created: 2013-08-10

from juriscraper.opinions.united_states.state import ark

## WARNING: THIS SCRAPER IS FAILING:
## This scraper is succeeding in development, but
## is failing in production.  We are not exactly
## sure why, and suspect that the hosting court
## site may be blocking our production IP and/or
## throttling/manipulating requests from production.


class Site(ark.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url_id = "courtofappeals"
        self.url = self.get_url()



"""Scraper for Supreme Court of U.S. OYEZ
CourtID: scotusoyez
Court Short Name: scotusoyez
Author: Andrei Chelaru
Reviewer:
Date created: 20 July 2014
"""

from datetime import datetime, date

from juriscraper.OralArgumentSite import OralArgumentSite


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        d = date.today()
        # page_nr can be between 0 and 5
        page_nr = 0
        self.url = "http://www.oyez.org/cases/{year}?page={nr}".format(
            year=d.year - 1, nr=page_nr
        )
        self.extender = {}

    def _get_download_urls(self):
        path = "//td[contains(concat(' ',@class,' '),' views-field-field-argument-value')][contains(., '/')]/preceding-sibling::td[2]/a/@href"
        download_urls = []
        for index, e in enumerate(self.html.xpath(path)):
            case_html = self._get_html_tree_by_url(e)
            path = "//a[contains(concat(' ',@class,' '),' arg-link audio') and contains(., 'Download')]/@href"
            urls = list(case_html.xpath(path))
            if len(urls) == 0:
                download_urls.append("")
                self.extender[index] = 1
            else:
                download_urls.extend(urls)
                self.extender[index] = len(urls)
        return download_urls

    def _get_case_names(self):
        path = "//td[contains(concat(' ',@class,' '),' views-field-field-argument-value')][contains(., '/')]/preceding-sibling::td[2]/a/text()"
        return list(self.html.xpath(path))

    def _get_case_dates(self):
        path = "//td[contains(concat(' ',@class,' '),' views-field-field-argument-value')][contains(., '/')]/span/text()"
        return map(self._return_case_date, self.html.xpath(path))

    @staticmethod
    def _return_case_date(e):
        e = "".join(e.split())
        return datetime.strptime(e, "%m/%d/%Y").date()

    def _get_docket_numbers(self):
        path = "//td[contains(concat(' ',@class,' '),' views-field-field-argument-value')][contains(., '/')]/preceding-sibling::td[1]/text()"
        return map(self._return_docket_number, self.html.xpath(path))

    @staticmethod
    def _return_docket_number(e):
        e = "".join(e.split())
        return e

    def _post_parse(self):
        self.docket_numbers = self._extend_result(self.docket_numbers)
        self.case_dates = self._extend_result(self.case_dates)
        self.case_names = self._extend_result(self.case_names)

    def _extend_result(self, result):
        new_result = []
        for index, nr in enumerate(self.extender.values()):
            new_result.extend([result[index]] * nr)
        return new_result



"""
Scraper for the United States Bankruptcy Appellate Panel for the Tenth Circuit
CourtID: bap10
Court Short Name: 10th Cir. BAP
Auth: Jon Andersen <janderse@gmail.com>
Reviewer: mlr
History:
    2014-09-01: First draft by Jon Andersen
    2014-09-02: Revised by mlr to use _clean_text() instead of pushing logic
                into the _get_case_dates function.
"""

from datetime import datetime
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www.bap10.uscourts.gov/opinions/new/opinion.txt"
        self.court_id = self.__module__
        self.my_case_names = []
        self.my_download_urls = []
        self.my_docket_numbers = []

        # HTTPS certificate is bad, but hopefully they'll fix it and we can remove the line below
        self.disable_certificate_verification()

    def _clean_text(self, text):
        """This page is a txt file, so here we convert it to something that
        can be easily ingested by lxml.

        Lines like:
          13-94.pdf|13|94|08/25/2014|Steve Christensen|Raymond Madsen|United States Bankruptcy Court for the District of Utah
        """
        # Nuke duplicates
        lines = set([line for line in text.split("\n") if line])

        # Build an XML tree.
        xml_text = "<rows>\n"
        for line in lines:
            values = line.split("|")
            xml_text += "  <row>\n"
            for value in values:
                xml_text += "    <value>%s</value>\n" % value
            xml_text += "  </row>\n"
        xml_text += "</rows>\n"
        return xml_text

    def _get_case_dates(self):
        path = "//row/value[4]/text()"
        return [
            datetime.strptime(date_string, "%m/%d/%Y").date()
            for date_string in self.html.xpath(path)
        ]

    def _get_case_names(self):
        plaintiffs = self.html.xpath("//row/value[6]/text()")
        defendants = self.html.xpath("//row/value[7]/text()")
        case_names = []
        for p, d in zip(plaintiffs, defendants):
            case_names.append("%s v. %s" % (p, d))
        return case_names

    def _get_download_urls(self):
        years = self.html.xpath("//row/value[2]/text()")
        file_names = self.html.xpath("//row/value[1]/text()")
        urls = []
        for y, f_n in zip(years, file_names):
            urls.append(
                "http://www.bap10.uscourts.gov/opinions/%s/%s" % (y, f_n)
            )
        return urls

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _get_docket_numbers(self):
        years = self.html.xpath("//row/value[2]/text()")
        numbers = self.html.xpath("//row/value[3]/text()")
        docket_numbers = []
        for y, n in zip(years, numbers):
            docket_numbers.append("%s-%s" % (y, n))
        return docket_numbers



# Author: Phil Ardery
# Contact: https://www.ndcourts.gov/contact-us
# Date created: 2019-02-28

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.exceptions import InsanityException
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.ndcourts.gov/supreme-court/recent-opinions?pageSize=100"
        self.cases = []

    def _process_html(self):
        for row in self.html.xpath('//table//div[@class="row"]'):
            case = self.case_fields_extract(row)
            self.case_fields_validate(case)
            case = self.case_fields_sanitize(case)
            self.cases.append(case)

    def case_fields_extract(self, row):
        text_lines = row.xpath("./div/p[1]/text()")
        text_lines = [
            l.strip() for l in text_lines if l.strip()
        ]  # Remove empty lines
        line_count = len(text_lines)
        return {
            "citation": text_lines[0],
            "docket": text_lines[1],
            "date": text_lines[2],
            "name": row.xpath(".//a")[0].text_content().strip(),
            "nature": text_lines[3],
            "judge": text_lines[4],
            "summary": " ".join(text_lines[5:line_count])
            if line_count > 5
            else "",
            "url": row.xpath(".//button/@onclick")[0].split("'")[1],
        }

    def case_fields_validate(self, case):
        if "ND" not in case["citation"]:
            raise InsanityException("Invalid citation: %s" % case["citation"])
        if not case["docket"].startswith("Docket No.:"):
            raise InsanityException(
                "Invalid docket raw string: %s" % case["docket"]
            )
        if not case["date"].startswith("Filing Date:"):
            raise InsanityException(
                "Invalid date string raw string: %s" % case["date"]
            )
        if not case["nature"].startswith("Case Type:"):
            raise InsanityException(
                "Invalid type raw string: %s" % case["nature"]
            )
        if not case["judge"].startswith("Author:"):
            raise InsanityException(
                "Invalid author raw string: %s" % case["judge"]
            )

    def case_fields_sanitize(self, case):
        for field in ["date", "docket", "judge", "nature"]:
            case[field] = case[field].split(":", 1)[1].strip()
        return case

    def _get_download_urls(self):
        return [case["url"] for case in self.cases]

    def _get_case_names(self):
        return [case["name"] for case in self.cases]

    def _get_case_dates(self):
        return [convert_date_string(case["date"]) for case in self.cases]

    def _get_docket_numbers(self):
        return [case["docket"] for case in self.cases]

    def _get_nature_of_suit(self):
        return [case["nature"] for case in self.cases]

    def _get_neutral_citations(self):
        return [case["citation"] for case in self.cases]

    def _get_judges(self):
        return [case["judge"] for case in self.cases]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.cases)

    def _get_summaries(self):
        return [case["summary"] for case in self.cases]



# Scraper for New York Appellate Divisions 1st Dept.
# CourtID: nyappdiv_1st
# Court Short Name: NY
# Author: Andrei Chelaru
# Reviewer: mlr
# Date: 2014-07-04

from datetime import date
from dateutil.rrule import rrule, MONTHLY

from juriscraper.AbstractSite import logger
from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.html_utils import get_html5_parsed_text
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        date_keys = rrule(
            MONTHLY, dtstart=date(2003, 11, 1), until=date(2015, 8, 30)
        )
        self.back_scrape_iterable = [i.date() for i in date_keys]
        self.row_base_path = '//tr[contains(./td[1]/a/@href, "3d")]'
        self.division = 1
        self.url = self.build_url()

    def _get_case_names(self):
        path = "%s/td[1]" % self.row_base_path
        return [cell.text_content() for cell in self.html.xpath(path)]

    def build_url(self, target_date=False):
        base = (
            "http://www.courts.state.ny.us/reporter/slipidx/aidxtable_%s"
            % self.division
        )
        if target_date:
            return "%s_%s_%s.shtml" % (
                base,
                target_date.year,
                target_date.strftime("%B"),
            )
        else:
            return "%s.shtml" % base

    def _get_download_urls(self):
        path = "%s/td[1]//a/@href" % self.row_base_path
        return self.html.xpath(path)

    def _get_case_dates(self):
        case_dates = []
        for element in self.html.xpath("//caption | //center"):
            date_string = (
                element.text_content().strip().replace("Cases Decided ", "")
            )
            path_prefix = (
                "./parent::"
                if element.tag == "caption"
                else "./following-sibling::"
            )
            path = path_prefix + "table[1]" + self.row_base_path
            cases = element.xpath(path)
            case_dates.extend([convert_date_string(date_string)] * len(cases))
        return case_dates

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_docket_numbers(self):
        path = "%s/td[3]" % self.row_base_path
        return map(
            self._add_str_to_list_where_empty_element, self.html.xpath(path)
        )

    def _get_judges(self):
        path = "%s/td[2]" % self.row_base_path
        return map(
            self._add_str_to_list_where_empty_element, self.html.xpath(path)
        )

    def _get_neutral_citations(self):
        path = "%s/td[4]" % self.row_base_path
        return [cell.text_content().strip() for cell in self.html.xpath(path)]

    @staticmethod
    def _add_str_to_list_where_empty_element(element):
        string_list = element.xpath("./text()")
        return string_list[0] if string_list else ""

    def _download_backwards(self, target_date):
        self.crawl_date = target_date
        logger.info("Running backscraper with date: %s" % target_date)
        self.url = self.build_url(target_date=target_date)
        self.html = self._download()

    def _make_html_tree(self, text):
        return get_html5_parsed_text(text)



"""
Scraper for Massachusetts Supreme Court
CourtID: mass
Court Short Name: MS
Author: Andrei Chelaru
Court Contact: SJCReporter@sjc.state.ma.us (617) 557-1030
Reviewer: mlr
History:
 - 2014-07-12: Created.
 - 2014-08-05, mlr: Updated regex.
 - 2014-09-18, mlr: Updated regex.
 - 2016-09-19, arderyp: Updated regex.
 - 2017-11-29, arderyp: Moved from RSS source to HTML
    parsing due to website redesign
"""

import re
import datetime

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import clean_string
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    """If you discover new examples that failed the
    regex below, please add them to the test_mass
    method in test_everything.py
    """

    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "https://www.mass.gov/service-details/new-opinions"
        self.court_id = self.__module__
        self.court_identifier = "SJC"
        # If you discover new examples that failed the
        # regex below, please add them to the test_mass
        # method in test_everything.py
        self.regex = "(.*)\s+\((.*)\)\s+\((.*)\)"
        self.year = datetime.datetime.now().year

    def _download(self, request_dict={}):
        """For tests, we must extract the year from the
        HTML instead of using the current year, otherwise
        test comparisons for example files created in
        past years will fail
        """
        html = super(Site, self)._download(request_dict)
        if self.test_mode_enabled():
            path = '//h2[contains(./text(), "Today\'s Published Opinions")]'
            header_text = html.xpath(path)[0].text_content()
            self.year = int(header_text.split(" ")[-1])
        self.set_local_variables()
        return html

    def set_local_variables(self):
        """The base xpath below requires '<current year>)' in
        the link text via the include_date definition. This
        means we might miss some opinions, when the court
        publishes line items without this pattern, but the
        only alternative it to email Mass every time there
        is an issue. Line items missing this pattern is not
        normal, but doesn't look altogether uncommon.
        Consequently, this approach is less than ideal, but
        it works for the time being.
        """
        exclude_list = "not(contains(., 'List of Un'))"
        include_date = "contains(., '%d)')" % self.year
        include_court = "contains(., '%s ') or contains(., '%s-')" % (
            self.court_identifier,
            self.court_identifier,
        )
        self.base_path = "//a/text()[%s][%s][%s]" % (
            exclude_list,
            include_date,
            include_court,
        )
        self.grouping_regex = re.compile(self.regex)

    def _get_case_names(self):
        names = []
        for s in self.html.xpath(self.base_path):
            s = clean_string(s)
            name_raw = self.grouping_regex.search(s).group(1)
            name = name_raw.replace(";", " / ")
            names.append(name)
        return names

    def _get_download_urls(self):
        path = "%s/parent::a/@href" % self.base_path
        return list(self.html.xpath(path))

    def _get_case_dates(self):
        dates = []
        for s in self.html.xpath(self.base_path):
            s = clean_string(s)
            date_string = self.grouping_regex.search(s).group(3)
            dates.append(convert_date_string(date_string))
        return dates

    def _get_docket_numbers(self):
        dockets = []
        for s in self.html.xpath(self.base_path):
            s = clean_string(s)
            docket_raw = self.grouping_regex.search(s).group(2)
            docket = docket_raw.replace(";", ",").replace(
                self.court_identifier + ",", self.court_identifier
            )
            dockets.append(docket)
        return dockets

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)



"""Scraper for the Vermont Environmental
CourtID: vt
Court Short Name: VT
Court Contact: submit form here https://www.vermontjudiciary.org/website-feedback-form
"""

from . import vt


class Site(vt.Site):
    def get_backscrape_max(self):
        return 1

    def get_division_id(self):
        return "4"



# Scraper for Florida 1st District Court of Appeal Per Curiam
# CourtID: flaapp1
# Court Short Name: flaapp1

from datetime import date

from juriscraper.AbstractSite import logger
from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court_number = 1
        self.type_id = 0  # Per Curiam
        self.url = self.get_url()
        self.back_scrape_iterable = [
            (12, 2012),
            (1, 2013),
            (2, 2013),
            (3, 2013),
            (4, 2013),
            (5, 2013),
            (6, 2013),
            (7, 2013),
            (8, 2013),
            (9, 2013),
            (10, 2013),
            (11, 2013),
            (12, 2013),
            (1, 2014),
            (2, 2014),
            (3, 2014),
            (4, 2014),
            (5, 2014),
            (6, 2014),
            (7, 2014),
            (8, 2014),
            (9, 2014),
            (10, 2014),
            (11, 2014),
            (12, 2014),
            (1, 2015),
            (2, 2015),
            (3, 2015),
            (4, 2015),
            (5, 2015),
            (6, 2015),
            (7, 2015),
            (8, 2015),
            (9, 2015),
            (10, 2015),
            (11, 2015),
            (12, 2015),
            (1, 2016),
            (2, 2016),
            (3, 2016),
            (4, 2016),
            (5, 2016),
            (6, 2016),
            (7, 2016),
            (8, 2016),
            (9, 2016),
            (10, 2016),
            (11, 2016),
            (12, 2016),
            (1, 2017),
            (2, 2017),
            (3, 2017),
            (4, 2017),
            (5, 2017),
            (6, 2017),
            (7, 2017),
            (8, 2017),
            (9, 2017),
            (10, 2017),
            (11, 2017),
            (12, 2017),
            (1, 2018),
            (2, 2018),
        ]

    def _get_case_names(self):
        return self.get_cell_content(3)

    def _get_download_urls(self):
        return self.get_cell_content(2, "//a/@href")

    def _get_case_dates(self):
        return [convert_date_string(ds) for ds in self.get_cell_content(6)]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _get_docket_numbers(self):
        return self.get_cell_content(1)

    def _get_dispositions(self):
        return self.get_cell_content(5)

    def _download_backwards(self, page_month_year):
        month = page_month_year[0]
        year = page_month_year[1]
        self.url = self.get_url(month, year)
        logger.info("Back scraping: %s" % self.url)
        self.html = self._download()

    def get_cell_content(self, cell_num, sub_path=False):
        path = '//table[@id="grdOpinions"]//tr/td[%d]' % cell_num
        cells = self.html.xpath(path + sub_path if sub_path else path)
        return (
            cells
            if sub_path
            else [cell.text_content().strip() for cell in cells]
        )

    def get_url(self, month=False, year=False):
        month = month if month else date.today().month
        year = year if year else date.today().year
        return (
            "https://edca.%ddca.org/Opinions.aspx?TypeID=%d&Day=All&Month=%d&Year=%d"
            % (self.court_number, self.type_id, month, year)
        )



#  Scraper for Kansas Appeals Court
# CourtID: kan_u


from juriscraper.opinions.united_states.state import kan_p


class Site(kan_p.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Unpublished"



from .string_utils import titlecase

titles = [
    "judge",
    "magistrate",
    "district",
    "chief",
    "senior",
    "bankruptcy",
    "mag.",
    "magistrate-judge",
    "mag/judge",
    "mag",
    "visiting",
    "special",
    "senior-judge",
    "master",
    "u.s.magistrate",
]
blacklist = [
    "a998",
    "agb",
    "am",
    "associated",
    "ca",
    "cases",
    "cet",
    "ch.",
    "cla",
    "clerk",
    "cp",
    "cvb",
    "db",
    "debt-magistrate",
    "discovery",
    "dj",
    "docket",
    "duty",
    "duty",
    "ec",
    "eck",
    "general",
    "grc",
    "gs",
    "hhl",
    "hon",
    "honorable",
    "inactive",
    "jne",
    "jv",
    "kec",
    "law",
    "lc",
    "llh",
    "lq",
    "maryland",
    "mediator",
    "merged",
    "mj",
    "mmh",
    "msh",
    "mwd",
    "no",
    "none",
    "prisoner",
    "pslc",
    "pro",
    "pso",
    "pt",
    "rmh",
    "se",
    "sf",
    "show",
    "successor",
    "u.s.",
    "tjc",
    "unassigned",
    "unassigned2",
    "unassigneddj",
    "unknown",
    "us",
    "usdc",
    "vjdistrict",
]

judge_normalizers = {
    # Generic Judge & Bankruptcy Judge we merge together for simplicity.
    "": "jud",
    "Judge": "jud",
    "Judge Judge": "jud",
    "District Judge": "jud",
    "Visiting Judge": "jud",
    "Bankruptcy": "jud",
    "Bankruptcy Judge": "jud",
    # Chief (normalize to jud for now, due to low sample size)
    "Chief": "jud",
    "Chief Judge": "jud",
    "Chief District Judge": "jud",
    # Magistrate
    "Mag": "mag",
    "Mag Judge": "mag",
    "mag/judge": "mag",
    "Magistrate": "mag",
    "Magistrate Judge": "mag",
    "Magistrate-Judge": "mag",
    "Magistrate Judge Mag": "mag",
    "Magistrate Judge Magistrate": "mag",
    "Magistrate Judge Magistrate Judge": "mag",
    # Chief Magistrate (normalize to magistrate for now, due to low sample size)
    "Chief Magistrate": "mag",
    "Chief Magistrate Judge": "mag",
    # Senior
    "Senior Judge": "ret-senior-jud",
    "Senior-Judge": "ret-senior-jud",
    # Special Master
    "Special Master": "spec-m",
    "Chief Special Master": "c-spec-m",
}


def normalize_judge_titles(title):
    """Normalize judge titles

    Take in a string like "Magistrate Judge" and return the normalized
    abbreviation from the POSITION_TYPES variable. Assumes that input is
    titlecased.

    Also normalizes things like:
     - District Judge --> Judge
     - Blank --> Judge
     - Bankruptcy Judge --> Judge
    """
    return judge_normalizers.get(title, "UNKNOWN: %s" % title)


def normalize_judge_names(name):
    """Cleans up little issues with people's names"""
    out = []
    words = name.split()
    for i, w in enumerate(words):
        # Michael J Lissner --> Michael J. Lissner
        if len(w) == 1 and w.isalpha():
            w = "%s." % w

        # Michael Lissner Jr --> Michael Lissner Jr.
        if w.lower() in ["jr", "sr"]:
            w = "%s." % w

        # J. Michael Lissner --> Michael Lissner
        # J. G. Lissner --> J. G. Lissner
        if i == 0 and w.lower() in ["j.", "j"]:
            next_word = words[i + 1]
            if not any(
                [
                    len(next_word) == 2 and next_word.endswith("."),
                    len(next_word) == 1,
                ]
            ):
                # Drop the word.
                continue
        out.append(w)

    return " ".join(out)


def normalize_judge_string(judge):
    """Split a string representing a judge returning their name and title.

    This code was generated and tested against all judge strings in the RECAP
    Archive.

    :param judge: A string representing a judge, e.g.
    :return: A tuple of the judge's name and their normalized position.

    >>> normalize_judge_string('Honorable Sue W. Wright')
    ('Sue W. Wright', 'jud')
    """
    judge = judge.replace(",", "")
    words = judge.lower().split()

    # Nuke bad junk (punct., j, blacklist, etc.)
    title_words = []
    name_words = []
    for w in words:
        contains_parens = "(" in w or ")" in w
        starts_poorly = w.startswith("-") or w.startswith("~")
        blacklisted = w in blacklist
        long_abbrev = len(w) > 2 and "." in w and w not in ["jr.", "sr."]
        if any([contains_parens, starts_poorly, blacklisted, long_abbrev]):
            continue
        if w in titles:
            title_words.append(w)
        else:
            name_words.append(w)

    title = normalize_judge_titles(titlecase(" ".join(title_words)))
    name = normalize_judge_names(titlecase(" ".join(name_words)))

    return name, title



"""Scraper for Mississippi Court of Appeals
CourtID: missctapp
Court Short Name: Miss. Ct. App.
"""

from juriscraper.opinions.united_states.state import miss


class Site(miss.Site):
    def get_court_parameter(self):
        return "COA"



"""
Scraper for Florida 3rd District Court of Appeal
CourtID: flaapp3
Court Short Name: flaapp3
Contact: 3dca@flcourts.org, Angel Falero <faleroa@flcourts.org> (305-229-6743)

History:
 - 2014-07-21: Written by Andrei Chelaru
 - 2014-07-24: Reviewed by mlr
 - 2015-07-28: Updated by m4h7
"""

from datetime import date

from juriscraper.lib.string_utils import titlecase, convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "http://www.3dca.flcourts.org/Opinions/ArchivedOpinions.shtml"
        )
        self.base_path = (
            "//h3/following::text()[.='OPINIONS']/following::table[1]//tr"
        )

    def _download(self, request_dict={}):
        html_l = super(Site, self)._download(request_dict)
        if self.test_mode_enabled():
            return [html_l]
        html_trees = []
        # this path reads the row for the last month in that year
        path_format = "//th[contains(., '{year}')]/following::tr[1]/td[position()>1]/a[contains(., '/')]/@href"
        path = path_format.format(year=date.today().year)
        for url in html_l.xpath(path):
            html_tree = self._get_html_tree_by_url(url, request_dict)
            html_trees.append(html_tree)
        return html_trees

    def _get_case_names(self):
        case_names = []  # return html_l
        for html_tree in self.html:
            case_names.extend(self._return_case_names(html_tree))
        return case_names

    def _return_case_names(self, html_tree):
        path = "%s/td[2]" % self.base_path
        cells = html_tree.xpath(path)
        names = [c.text_content().strip() for c in cells]

        # Return formatted text for non-empty cells
        return [titlecase(n.lower()) for n in names if n]

    def _get_download_urls(self):
        download_urls = []
        for html_tree in self.html:
            download_urls.extend(self._return_download_urls(html_tree))
        return download_urls

    def _return_download_urls(self, html_tree):
        path = "%s/td[1]//a/@href" % self.base_path
        return html_tree.xpath(path)

    def _get_case_dates(self):
        case_dates = []
        for html_tree in self.html:
            case_dates.extend(self._return_dates(html_tree))
        return case_dates

    def _return_dates(self, html_tree):
        path_date_string = "//h3/text()"
        text = html_tree.xpath(path_date_string)[0]
        date_string = text.split()[0]
        case_date = convert_date_string(date_string)

        # Count rows with non-empty first cells
        path_first_cell = "%s/td[1]" % self.base_path
        first_cells = html_tree.xpath(path_first_cell)
        opinion_count = sum(1 for r in first_cells if r.text_content().strip())

        return [case_date] * opinion_count

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _get_docket_numbers(self):
        docket_numbers = []
        for html_tree in self.html:
            docket_numbers.extend(self._return_docket_numbers(html_tree))
        return docket_numbers

    def _return_docket_numbers(self, html_tree):
        path = "%s/td[1]//a" % self.base_path
        return [a.text_content().strip() for a in html_tree.xpath(path)]



"""This site is pretty bad. Very little HTML; everything is separated by
<br> tags. There is currently a case at the bottom of the page from 2009
that has incomplete meta data. You can see it in the example document.
"""

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www2.ca3.uscourts.gov/recentop/week/recprec.htm"
        self.court_id = self.__module__

    def _get_case_names(self):
        return [
            e
            for e in self.html.xpath('//a[contains(@href, "opinarch")]/text()')
        ]

    def _get_download_urls(self):
        return [
            e
            for e in self.html.xpath('//a[contains(@href, "opinarch")]/@href')
        ]

    def _get_case_dates(self):
        dates = []
        for text_string in self.html.xpath("//text()"):
            if not text_string.lower().startswith("filed"):
                continue
            else:
                date_string = text_string.split(" ")[1]
                date_string = date_string.strip().strip(",")
                dates.append(convert_date_string(date_string))
        return dates

    def _get_docket_numbers(self):
        docket_numbers = []
        for text_string in self.html.xpath("//text()"):
            if not text_string.lower().startswith("filed"):
                continue
            else:
                docket_numbers.append(text_string.split(" ")[3])
        return docket_numbers

    def _get_precedential_statuses(self):
        statuses = []
        for _ in range(0, len(self.case_names)):
            if "recprec" in self.url:
                statuses.append("Published")
            elif "recnonprec" in self.url:
                statuses.append("Unpublished")
            else:
                statuses.append("Unknown")
        return statuses

    def _get_lower_courts(self):
        lower_courts = []
        for e in self.html.xpath('//a[contains(@href, "opinarch")]'):
            text_strings = e.xpath("./following-sibling::text()[1]")
            text_string = " ".join(text_strings)
            if (
                text_string.lower().startswith("filed")
                or text_string.strip() == ""
            ):
                lower_courts.append("Unknown")
            else:
                lower_courts.append(text_string.strip())
        return lower_courts



from juriscraper.opinions.united_states_backscrapers.state import (
    idahoctapp_civil,
)


class Site(idahoctapp_civil.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www.isc.idaho.gov/opinions/cacrim.htm"
        self.court_id = self.__module__



# Scraper for the United States Court of International Trade
# CourtID: cit
# Court Short Name: Ct. Int'l Trade
# Neutral Citation Format: Ct. Int'l Trade No. 12-1
from juriscraper.OpinionSite import OpinionSite
import re
import time
from datetime import date
from lxml import html


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        # This is a special backscraper to deal with problems on the 2004 page.
        self.url = "http://www.cit.uscourts.gov/SlipOpinions/SlipOps-2004.html"
        self.court_id = self.__module__

    def _get_download_urls(self):
        return [
            t
            for t in self.html.xpath(
                "//table[3]//tr[position() > 1]/td/font//a/@href"
            )
        ]

    def _get_neutral_citations(self):
        neutral_citations = []
        for e in self.html.xpath(
            "//table[3]//tr[position() > 1]/td[1]/font//a"
        ):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            neutral_citations.append(s)
        return neutral_citations

    def _get_case_names(self):
        case_names = []
        for e in self.html.xpath("//table[3]//tr[position() > 1]/td[2]/*"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            # We strip "erratum: mm/dd/yyyy" from the case names of errata docs.
            if "erratum" in s:
                case_names.append(s.strip()[:-18])
            else:
                case_names.append(s.strip())
        return case_names

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_case_dates(self):
        case_dates = []
        for e in self.html.xpath("//table[3]//tr[position() > 1]/td[3]/font"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            if s == "06/012004":
                case_dates.append(
                    date.fromtimestamp(
                        time.mktime(
                            time.strptime("06/01/2004".strip(), "%m/%d/%Y")
                        )
                    )
                )
            else:
                case_dates.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime(s.strip(), "%m/%d/%Y"))
                    )
                )
        return case_dates

    # Because there can be multiple docket numbers we have to replace some newlines.
    def _get_docket_numbers(self):
        docket_numbers = []
        for e in self.html.xpath("//table[3]//tr[position() > 1]/td[4]/font"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            docket_numbers.append(s.replace("\r\n", " &"))
        return docket_numbers



# History:
# - Long ago: Created by mlr
# - 2014-11-07: Updated by mlr to use new website.

from datetime import datetime

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www.ca5.uscourts.gov/rss.aspx?Feed=Opinions&Which=All&Style=Detail"
        self.court_id = self.__module__

    def _get_case_names(self):
        path = "//item/description/text()[2]"
        return [s for s in self.html.xpath(path)]

    def _get_download_urls(self):
        path = "//item/link"
        return [e.tail for e in self.html.xpath(path)]

    def _get_case_dates(self):
        path = "//item/description/text()[5]"
        return [
            datetime.strptime(date_string, "%m/%d/%Y").date()
            for date_string in self.html.xpath(path)
        ]

    def _get_docket_numbers(self):
        path = "//item/description/text()[1]"
        return [s for s in self.html.xpath(path)]

    def _get_precedential_statuses(self):
        path = "//item/description/text()[3]"
        statuses = []
        for status in self.html.xpath(path):
            if status == "pub":
                statuses.append("Published")
            elif status == "unpub":
                statuses.append("Unpublished")
            else:
                statuses.append("Unknown")
        return statuses

    def _get_nature_of_suit(self):
        path = "//item/description/text()[4]"
        return [s for s in self.html.xpath(path)]



# Author: Michael Lissner
# Date created: 2013-06-11

import re
from datetime import datetime
from lxml import html
from time import sleep
from juriscraper.OpinionSiteWebDriven import OpinionSiteWebDriven
from juriscraper.lib.string_utils import titlecase


class Site(OpinionSiteWebDriven):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.sdjudicial.com/sc/scopinions.aspx"
        self.uses_selenium = True
        self.year = False

    def _get_download_urls(self):
        path = '//tr[contains(@id, "ctl00xmainCopyxWGOpinions_r")]//a[2]/@href'
        return list(self.html.xpath(path))

    def _get_case_names(self):
        if self.year == 2005:
            strings = []
            path = '//tr[contains(@id, "ctl00xmainCopyxWGOpinions_r")]/td[2]'
            for e in self.html.xpath(path):
                strings.append(
                    html.tostring(e, method="text", encoding="unicode")
                )
        else:
            path = '//tr[contains(@id, "ctl00xmainCopyxWGOpinions_r")]/td[2]/text()'
            strings = list(self.html.xpath(path))
        case_names = []
        for s in strings:
            try:
                case_name = re.search(
                    r"(.*)(\d{4} S\.?D\.? \d{1,4})", s, re.MULTILINE
                ).group(1)
                case_names.append(titlecase(case_name.lower()))
            except AttributeError:
                print("AttributeError on: %s" % titlecase(s))
                if "myrl" in s.lower() and self.year == 2000:
                    case_names.append("Lends His Horse v. Myrl & Roy")
                elif "springer" in s.lower() and self.year == 2000:
                    case_names.append("State v. Springer-Ertl")
                elif (
                    "formatting provided courtesy" in s.lower()
                    and self.year == 2000
                ):
                    case_names.append("Lois F. Henry v. Harold L. Henry")
                elif "spec. v. avera" in s.lower() and self.year == 2001:
                    case_names.append(
                        "Drs., Residents, and Orth. Surg. Spec. v. Avera St. Luke"
                    )
                elif "clausen" in s.lower() and self.year == 2003:
                    case_names.append(
                        "Kelly Clausen v. Northern Plains Recycling, Fireman"
                    )
                elif "burkhart" in s.lower() and self.year == 2003:
                    case_names.append("Burkhart v. Lillehaug and Lillihaug")
                elif "bennett" in s.lower() and self.year == 2003:
                    case_names.append(
                        "State of South Dakota, ex rel., Megan Bennett v. Thomas G. Peterson"
                    )
                elif "duane" in s.lower() and self.year == 2004:
                    case_names.append(
                        "State of South Dakota v. Duane J. St. John"
                    )
                elif "square" in s.lower() and self.year == 2005:
                    case_names.append(
                        "Town Square Limited Partnership v. Clay County Board of Equalization"
                    )
                elif "toft" in s.lower() and self.year == 2006:
                    case_names.append("Toft v. Toft & Stratmeyer")
                elif "masteller" in s.lower() and self.year == 2006:
                    case_names.append("Masteller v. Champion Home")
                elif "nuzum" in s.lower() and self.year == 2006:
                    case_names.append("State v. Nuzum")
                elif "aaberg" in s.lower() and self.year == 2006:
                    case_names.append("State v. Aaberg")
                elif "bryan" in s.lower() and self.year == 2006:
                    case_names.append("O'Bryan v. Ashland")
                elif "j.d.m.c." in s.lower() and self.year == 2007:
                    case_names.append("In Re the Matter of J.D.M.C.")
                elif "argus" in s.lower() and self.year == 2007:
                    case_names.append("Argus Leader v. Hagen")
                elif "nedved" in s.lower() and self.year == 2007:
                    case_names.append(
                        "Countryside S. Homeowners Assoc. v. Nedved"
                    )
                elif "gisi" in s.lower() and self.year == 2007:
                    case_names.append("Gisi v. Gisi")
                elif "western" in s.lower() and self.year == 2007:
                    case_names.append("Great Western Bank v. H&E Enterprises")
                elif "canyon" in s.lower() and self.year == 2007:
                    case_names.append("Jackson v. Canyon Place Homeowners")
                elif "vansteenwyk" in s.lower() and self.year == 2007:
                    case_names.append("Vanteenwyk v. Baumgartner Trees")
                elif "jewett" in s.lower() and self.year == 2011:
                    case_names.append("Jewett v. Real Tuff")
                elif "jensen" in s.lower() and self.year == 2011:
                    case_names.append("State v. Jensen")
                elif "cook" in s.lower() and self.year == 2011:
                    case_names.append("Orr v. Cook")
                elif "living" in s.lower() and self.year == 2011:
                    case_names.append("Benson Living Trust")
                else:
                    raise AttributeError
        return case_names

    def _get_case_dates(self):
        path = '//tr[contains(@id, "ctl00xmainCopyxWGOpinions_r")]/td[1]/@uv'
        return [
            datetime.strptime(date_string, "%m/%d/%Y").date()
            for date_string in self.html.xpath(path)
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_docket_numbers(self):
        path = '//tr[contains(@id, "ctl00xmainCopyxWGOpinions_r")]//a[2]/@href'
        docket_numbers = []
        for url in self.html.xpath(path):
            try:
                # New style: easy
                docket_numbers.append(re.search(r"(\d+).pdf", url).group(1))
            except AttributeError:
                docket_numbers.append(None)
        return docket_numbers

    def _get_neutral_citations(self):
        if self.year == 2005:
            strings = []
            path = '//tr[contains(@id, "ctl00xmainCopyxWGOpinions_r")]/td[2]'
            for e in self.html.xpath(path):
                strings.append(
                    html.tostring(e, method="text", encoding="unicode")
                )
        else:
            path = '//tr[contains(@id, "ctl00xmainCopyxWGOpinions_r")]/td[2]/text()'
            strings = list(self.html.xpath(path))
        neutral_cites = []
        for s in strings:
            try:
                neutral_cite = re.search(
                    r"(.*)(\d{4} S\.?D\.? \d{1,4})", s, re.MULTILINE
                ).group(2)
                neutral_cites.append(titlecase(neutral_cite))
            except AttributeError:
                if "myrl" in s.lower() and self.year == 2000:
                    neutral_cites.append("2000 SD 146")
                elif "springer" in s.lower() and self.year == 2000:
                    neutral_cites.append("2000 SD 56")
                elif (
                    "formatting provided courtesy" in s.lower()
                    and self.year == 2000
                ):
                    neutral_cites.append("2000 SD 4")
                elif "spec. v. avera" in s.lower() and self.year == 2001:
                    neutral_cites.append("2001 SD 9")
                elif "clausen" in s.lower() and self.year == 2003:
                    neutral_cites.append("2003 SD 63")
                elif "burkhart" in s.lower() and self.year == 2003:
                    neutral_cites.append("2003 SD 62")
                elif "bennett" in s.lower() and self.year == 2003:
                    neutral_cites.append("2003 SD 16")
                elif "duane" in s.lower() and self.year == 2004:
                    neutral_cites.append("2004 SD 15")
                elif "square" in s.lower() and self.year == 2005:
                    neutral_cites.append("2005 SD 99")
                elif "toft" in s.lower() and self.year == 2006:
                    neutral_cites.append("2006 SD 91")
                elif "masteller" in s.lower() and self.year == 2006:
                    neutral_cites.append("2006 SD 90")
                elif "nuzum" in s.lower() and self.year == 2006:
                    neutral_cites.append("2006 SD 89")
                elif "aaberg" in s.lower() and self.year == 2006:
                    neutral_cites.append("2006 SD 58")
                elif "bryan" in s.lower() and self.year == 2006:
                    neutral_cites.append("2006 SD 56")
                elif "j.d.m.c." in s.lower() and self.year == 2007:
                    neutral_cites.append("2007 SD 97")
                elif "argus" in s.lower() and self.year == 2007:
                    neutral_cites.append("2007 SD 96")
                elif "nedved" in s.lower() and self.year == 2007:
                    neutral_cites.append("2007 SD 70")
                elif "gisi" in s.lower() and self.year == 2007:
                    neutral_cites.append("2007 SD 39")
                elif "western" in s.lower() and self.year == 2007:
                    neutral_cites.append("2007 SD 38")
                elif "canyon" in s.lower() and self.year == 2007:
                    neutral_cites.append("2007 SD 37")
                elif "vansteenwyk" in s.lower() and self.year == 2007:
                    neutral_cites.append("2007 SD 36")
                elif "jewett" in s.lower() and self.year == 2011:
                    neutral_cites.append("2011 S.D.33")
                elif "jensen" in s.lower() and self.year == 2011:
                    neutral_cites.append("2011 S.D.32")
                elif "cook" in s.lower() and self.year == 2011:
                    neutral_cites.append("2011 S.D.31")
                elif "living" in s.lower() and self.year == 2011:
                    neutral_cites.append("2011 S.D.30")
                else:
                    raise AttributeError
        return neutral_cites

    def _download_backwards(self, year):
        self.year = year
        self.initiate_webdriven_session()
        elems = self.webdriver.find_elements_by_class_name("igeb_ItemLabel")
        elem = [elem for elem in elems if elem.text == str(year)][0]
        elem.click()
        sleep(5)

        text = self.webdriver.page_source
        html_tree = html.fromstring(text)
        html_tree.make_links_absolute(self.url)

        remove_anchors = lambda url: url.split("#")[0]
        html_tree.rewrite_links(remove_anchors)

        self.html = html_tree
        self.status = 200
        self.webdriver.quit()



# Scraper for the United States Court of International Trade
# CourtID: cit
# Court Short Name: Ct. Int'l Trade
# Neutral Citation Format: Ct. Int'l Trade No. 12-1
from juriscraper.OpinionSite import OpinionSite
import re
import time
from datetime import date
from lxml import html


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        # This is a special backscraper to deal with problems on the 2008 page.
        self.url = "http://www.cit.uscourts.gov/SlipOpinions/SlipOps-2008.html"
        self.court_id = self.__module__

    def _get_download_urls(self):
        return [
            t
            for t in self.html.xpath(
                "//table[3]/tr[position() > 1]/td[1]/a/@href | //table[3]/tr[position() > 1]/td[1]/font/a/@href"
            )
        ]

    def _get_neutral_citations(self):
        neutral_citations = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[1]"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            neutral_citations.append(s)
        return neutral_citations

    def _get_case_names(self):
        case_names = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[2]"):
            t = html.tostring(e, method="text", encoding="unicode")
            # We strip "Errata: mm/dd/yyyy" from the case names of errata docs.
            if "Errata" in t:
                case_names.append(t.strip()[:-18])
            # We strip "Public version posted on mm/dd/yyyy" from some case names.
            elif "posted on" in t:
                case_names.append(t.strip()[:-34])
            else:
                case_names.append(t.strip())
        return case_names

    def _get_precedential_statuses(self):
        statuses = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[2]"):
            s = (
                html.tostring(e, method="text", encoding="unicode")
                .lower()
                .strip()
            )
            if "errata" in s:
                statuses.append("Errata")
            else:
                statuses.append("Published")
        return statuses

    # This does not capture the release dates for the errata documents.
    # The errata release date is listed in column 2. This will use the original
    # release date instead.
    def _get_case_dates(self):
        case_dates = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[3]"):
            if (
                html.tostring(e, method="text", encoding="unicode").strip()
                == "08/05/20008"
            ):
                date_string = "08/05/2008"
                case_dates.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime(date_string, "%m/%d/%Y"))
                    )
                )
            else:
                date_string = html.tostring(
                    e, method="text", encoding="unicode"
                ).strip()
                case_dates.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime(date_string, "%m/%d/%Y"))
                    )
                )
        return case_dates

    # Because there can be multiple docket numbers we have to replace some newlines.
    def _get_docket_numbers(self):
        docket_numbers = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[4]"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            docket_numbers.append(s.replace("\r\n", " &"))
        return docket_numbers

    # Comment out the following section if backscraping years 1999-2005.
    # Because there are sometimes multiple judges we have to strip some whitespace.
    def _get_judges(self):
        judges = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[5]"):
            s = html.tostring(e, method="text", encoding="unicode")
            judges.append(s.strip())
        return judges



# Author: Michael Lissner
# History:
# - 2013-06-11: Birth.
# - 2013-08-06: Revised by Brian Carver
# - 2014-08-05: Updated URL by mlr

import re
from lxml import html

from juriscraper.AbstractSite import logger
from juriscraper.OpinionSiteWebDriven import OpinionSiteWebDriven
from juriscraper.lib.html_utils import fix_links_in_lxml_tree
from juriscraper.lib.string_utils import convert_date_string, titlecase


class Site(OpinionSiteWebDriven):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://ujs.sd.gov/Supreme_Court/opinions.aspx"
        self.back_scrape_iterable = [
            (0, 2014),
            (1, 2014),
            (2, 2014),
            (3, 2014),
            (0, 2013),
            (1, 2013),
            (2, 2013),
            (3, 2013),
            (4, 2013),
            (5, 2013),
            (6, 2013),
        ]
        self.uses_selenium = True
        regex_year_group = "(\d{4} S\.?D\.? \d{1,4})"
        self.regex = "(.*)%s" % regex_year_group
        # The court decided to publish 5 records on January 24th 2018
        # in an alternate format where the neutral citation appears
        # before the case name, instead of vice versa.  We contacted the
        # court multiple times, but they wouldn't respond, so we had to
        # add the additional regex below to handle this use case.
        self.regex_alt = "%s, (.*)" % regex_year_group
        self.base_path = "//table[@id='ContentPlaceHolder1_PageContent_gvOpinions']//tr[position()>1]/td"

    def _get_download_urls(self):
        path = "%s//a/@href[contains(.,'pdf')]" % self.base_path
        return list(self.html.xpath(path))

    def _get_case_names(self):
        path = "%s/a[contains(@href, 'pdf')]/text()" % self.base_path
        case_names = []
        for s in self.html.xpath(path):
            case_name = self.extract_regex_group(1, s)
            if not case_name:
                case_name = self.extract_regex_group(2, s, True)
            case_names.append(titlecase(case_name.upper()))
        return case_names

    def _get_case_dates(self):
        path = "%s[1]/text()" % self.base_path
        return [convert_date_string(ds) for ds in self.html.xpath(path)]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_neutral_citations(self):
        path = "%s/a[contains(@href, 'pdf')]/text()" % self.base_path
        neutral_cites = []
        for s in self.html.xpath(path):
            neutral_cite = self.extract_regex_group(2, s)
            if not neutral_cite:
                neutral_cite = self.extract_regex_group(1, s, True)
            # Make the citation SD instead of S.D. The former is a neutral cite, the latter, the South Dakota Reporter
            neutral_cites.append(neutral_cite.replace(".", "").upper())
        return neutral_cites

    def _download_backwards(self, page_year):
        logger.info("Running with params: %s" % (page_year,))
        self.initiate_webdriven_session()

        # Select the year (this won't trigger a GET unless it's changed)
        path = "//*[@id='ContentPlaceHolder1_PageContent_OpinionYears']/option[@value={year}]".format(
            year=page_year[1]
        )
        option = self.webdriver.find_element_by_xpath(path)
        option.click()

        if page_year[0] != 0:
            # Not the first, page, go to the one desired.
            links = self.webdriver.find_elements_by_xpath(
                "//a[@href[contains(., 'Page')]]"
            )
            links[page_year[0] - 1].click()

        text = self._clean_text(self.webdriver.page_source)
        self.webdriver.quit()
        html_tree = html.fromstring(text)

        html_tree.rewrite_links(
            fix_links_in_lxml_tree, base_href=self.request["url"]
        )
        self.html = html_tree
        self.status = 200

    def extract_regex_group(self, group, string, alt=False):
        regex = self.regex_alt if alt else self.regex
        return re.search(regex, string, re.MULTILINE).group(group)



from juriscraper.opinions.united_states.federal_appellate import (
    scotus_chambers,
)


class Site(scotus_chambers.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.precedential = "Relating-to"
        self.court = "relatingtoorders"



from ..lib.log_tools import make_default_logger
from ..lib.utils import clean_court_object
from datetime import datetime, timedelta
from dateutil.rrule import rrule, WEEKLY

logger = make_default_logger()


class LASCSearch(object):
    """
    An object designed to search the LA Superior Court Media Access Portal
    (MAP). It searches by date, looks up individual cases, and collects PDFs on
    those cases if made available in the MAP.
    """

    def __init__(self, session):
        self.session = session
        self.api_base = "media.lacourt.org/api/AzureApi/"

    def get_json_from_internal_case_id(self, internal_case_id):
        """Query LASC for the case json based on the internal case id

        :param internal_case_id: An internal ID of the form, 19STCV25157;SS;CV
        :return: The parsed docket data for the case requested
        """
        r = self.session.get(
            "https://%sGetCaseDetail/%s" % (self.api_base, internal_case_id)
        )
        self._check_success(r)

        return self._parse_case_data(r.json())

    def query_cases_by_date(self, start, end):
        """Query LASC for a list of cases between two dates (inclusive)

        The LASC interface only allows queries of up to seven days at a time.
        If a wider request is made, break it into multiple smaller requests
        and append their results together.

        :param start: A date object to start the query
        :param end: A date object to end the query
        :return: A list of case metadata objects
        """
        start_str = start.strftime("%m-%d-%Y")
        end_str = end.strftime("%m-%d-%Y")
        date_query_url = "https://%sGetRecentCivilCases/%s/%s" % (
            self.api_base,
            start_str,
            end_str,
        )
        r = self.session.get(date_query_url)
        cases = r.json()["ResultList"]

        # Normalize the date data
        normal_cases = []
        for case in cases:
            clean_case = {
                "internal_case_id": case["InternalCaseID"],
                "judge_code": case["JudgeCode"] or "",
                "case_type_code": case["CaseTypeCode"] or "",
            }
            normal_cases.append(clean_case)

        return normal_cases

    def get_pdf_from_url(self, pdf_url):
        """Get a PDF from the MAP

        :param pdf_url: The URL to a particular PDF you wish to download
        :return: The binary PDF data
        """

        logger.info(u"Api ViewDocument called.  Downloading PDF ")
        return self.session.get(pdf_url).content

    @staticmethod
    def _parse_case_data(case_data):
        """
        This function normalizes the json docket data we get from LA MAP

        The MAP, mercifully, gives us JSON data, so this isn't too bad. The
        main task, then, is to convert the various field names it provides over
        to field names following our conventions.

        :param case_data: A python dict of a docket as gathered from the MAP
        (this will be JSON data in the MAP, but convert it to Python before
        calling this method.)
        :return: The parsed, cleaned, normalized data
        """
        logger.info(
            u"Parsing lasc data from returned JSON into normalized " u"format"
        )

        try:
            data = case_data["ResultList"][0]["NonCriminalCaseInformation"]
        except IndexError:
            # Some cases don't have any data at all, see: P4160;JCC;CV. When
            # this happens, Juriscraper returns nothing as well.
            return {}

        # Docket Normalization
        clean_data = {u"QueuedCase": [], u"LASCPDF": [], u"LASCJSON": []}

        case_info = data[u"CaseInformation"]

        docket = {
            u"date_filed": case_info[u"FilingDate"].date(),
            u"date_disposition": None,
            u"docket_number": case_info[u"CaseNumber"],
            u"district": case_info[u"District"],
            u"division_code": case_info[u"DivisionCode"],
            u"disposition_type": case_info[u"DispositionType"],
            u"disposition_type_code": u"",
            u"case_type_str": case_info[u"CaseTypeDescription"],
            u"case_type_code": case_info[u"CaseType"],
            u"case_name": case_info[u"CaseTitle"] or u"",
            u"judge_code": u"",
            u"judge_name": case_info[u"JudicialOfficer"],
            u"courthouse_name": case_info[u"Courthouse"],
        }

        if case_info[u"StatusDate"] is not None:
            docket["date_status"] = datetime.strptime(
                case_info["StatusDate"], "%m/%d/%Y"
            ).date()
        else:
            docket[u"date_status"] = case_info[u"StatusDate"]

        docket[u"status_code"] = case_info[u"StatusCode"] or ""
        docket[u"status_str"] = case_info[u"Status"]
        clean_data[u"Docket"] = docket

        # Register of Actions normalization
        clean_data[u"Action"] = []
        for action in data[u"RegisterOfActions"]:
            registered_action = {
                u"date_of_action": action[u"RegisterOfActionDate"],
                u"description": action[u"Description"],
                u"additional_information": action[u"AdditionalInformation"],
            }
            clean_data[u"Action"].append(registered_action)

        # Cross References normalization
        clean_data[u"CrossReference"] = []
        for cross_ref in data[u"CrossReferences"]:
            cross_reference = {
                u"date_cross_reference": cross_ref[u"CrossReferenceDate"],
                u"cross_reference_docket_number": cross_ref[
                    u"CrossReferenceCaseNumber"
                ]
                or u"",
                u"cross_reference_type": cross_ref[
                    u"CrossReferenceTypeDescription"
                ],
            }
            clean_data[u"CrossReference"].append(cross_reference)

        # Documents Filed
        clean_data[u"DocumentFiled"] = []
        for doc_filed in data[u"DocumentsFiled"]:
            document = {
                u"date_filed": doc_filed[u"DateFiled"],
                u"memo": doc_filed[u"Memo"] or "",
                u"document_type": doc_filed[u"Document"] or "",
                u"party_str": doc_filed[u"Party"] or "",
            }
            clean_data[u"DocumentFiled"].append(document)

        clean_data[u"QueuedPDF"] = []
        clean_data[u"DocumentImage"] = []
        for doc_image in data[u"DocumentImages"]:
            image = {
                u"date_processed": doc_image[u"createDate"],
                u"date_filed": doc_image[u"docFilingDate"],
                u"doc_id": doc_image[u"docId"],
                u"page_count": doc_image[u"pageCount"],
                u"document_type": doc_image[u"documentType"] or "",
                u"document_type_code": doc_image[u"documentTypeID"],
                u"image_type_id": doc_image[u"imageTypeId"],
                u"app_id": doc_image[u"appId"] or u"",
                u"odyssey_id": doc_image[u"OdysseyID"] or "",
                u"is_downloadable": doc_image[u"IsDownloadable"],
                u"security_level": doc_image[u"securityLevel"],
                u"description": doc_image[u"description"] or "",
                u"doc_part": doc_image[u"docPart"] or "",
                u"is_available": False,
            }

            pdf_queue = {
                u"internal_case_id": data[u"CaseInformation"][u"CaseID"],
                u"document_id": doc_image[u"docId"],
            }

            clean_data[u"DocumentImage"].append(image)
            clean_data[u"QueuedPDF"].append(pdf_queue)

        clean_data[u"Party"] = []
        for party in data[u"Parties"]:
            party_obj = {
                u"attorney_name": party[u"AttorneyName"],
                u"attorney_firm": party[u"AttorneyFirm"],
                u"entity_number": party[u"EntityNumber"],
                u"party_name": party[u"Name"],
                u"party_flag": party[u"PartyFlag"],
                u"party_type_code": party[u"PartyTypeCode"] or "",
                u"party_description": party[u"PartyDescription"] or "",
            }
            clean_data["Party"].append(party_obj)

        clean_data[u"Proceeding"] = []
        for proceeding in data[u"PastProceedings"]:
            event = {
                u"past_or_future": 1,
                u"date_proceeding": proceeding[u"ProceedingDate"],
                u"proceeding_time": proceeding[u"ProceedingTime"],
                u"proceeding_room": proceeding[u"ProceedingRoom"],
                u"am_pm": proceeding[u"AMPM"],
                u"memo": proceeding[u"Memo"],
                u"courthouse_name": proceeding[u"CourthouseName"],
                u"address": proceeding[u"Address"],
                u"result": proceeding[u"Result"],
                u"judge_name": proceeding[u"Judge"],
                u"event": proceeding[u"Event"],
            }
            clean_data[u"Proceeding"].append(event)

        for proceeding in data[u"FutureProceedings"]:
            event = {
                u"past_or_future": 2,
                u"date_proceeding": proceeding[u"ProceedingDate"],
                u"proceeding_time": proceeding[u"ProceedingTime"],
                u"proceeding_room": proceeding[u"ProceedingRoom"],
                u"am_pm": proceeding[u"AMPM"],
                u"memo": proceeding[u"Memo"],
                u"courthouse_name": proceeding[u"CourthouseName"],
                u"address": proceeding[u"Address"],
                u"result": proceeding[u"Result"],
                u"judge_name": proceeding[u"Judge"],
                u"event": proceeding[u"Event"],
            }
            clean_data[u"Proceeding"].append(event)

        clean_data[u"TentativeRuling"] = []
        for ruling in data[u"TentativeRulings"]:
            tentative_ruling = {
                u"date_created": ruling[u"CreationDate"],
                u"date_hearing": ruling[u"HearingDate"],
                u"department": ruling[u"Department"],
                u"ruling": ruling[u"Ruling"],
            }

            clean_data[u"TentativeRuling"].append(tentative_ruling)

        clean_data = clean_court_object(clean_data)

        return clean_data

    def _check_success(self, r):
        if r.json()["IsSuccess"]:
            self.case_data = r.text
            logger.info(u"Successful query into LASC map")
        else:
            logger.info(u"Failure to query into LASC map")



"""
History:
 - 2014-08-07: Fixed due to InsanityError on docketnumber
"""

from datetime import date

from juriscraper.opinions.united_states.federal_appellate import ca9_p


class Site(ca9_p.Site):
    """The unpublished cases have one more column than the published. Thus some
    overriding is done here. More than usual, but it's very slight tweaks."""

    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www.ca9.uscourts.gov/memoranda/?o_mode=view&amp;o_sort_field=21&amp;o_sort_type=DESC&o_page_size=100"
        self.court_id = self.__module__
        self.position = ""
        self.base = self.get_base_path()
        self.back_scrape_date_start = date(2009, 11, 11)
        self.back_scrape_date_end = date(2015, 1, 1)
        self.back_scrape_iterable = self.get_backscrape_iterable()
        self.back_scrape_url = "http://www.ca9.uscourts.gov/memoranda/"

        def get_back_scrape_parameters(self, d):
            return {
                "c_page_size": "50",
                "c__ff_cms_memoranda_case_name_operator": "like",
                "c__ff_cms_memoranda_case_num_operator": "like",
                "c__ff_cms_memoranda_case_panel_operator": "like",
                "c__ff_cms_memoranda_case_origin_operator": "like",
                "c__ff_cms_memoranda_case_type_operator": "%3D",
                "c__ff_cms_memoranda_filed_date_operator": "like",
                "c__ff_cms_memoranda_filed_date": d.strftime("%m/%d/%Y"),
                "c__ff_onSUBMIT_FILTER": "Search",
            }



"""
Backscraper for Indiana Tax Court
CourtID: indtc
Court Short Name: Ind. Tax.
Auth: Jon Andersen <janderse@gmail.com>
Reviewer: mlr
History:
    2014-09-03: Created by Jon Andersen
"""
from juriscraper.opinions.united_states_backscrapers.state import ind


class Site(ind.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.in.gov/judiciary/opinions/archtax.html"



"""Scraper for Mississippi Court of Appeals
CourtID: missctapp
Court Short Name: Miss. Ct. App.
Author: Jon Andersen
Reviewer: mlr
Type: Precedential
History:
    2014-09-21: Created by Jon Andersen
"""

from datetime import date

from juriscraper.opinions.united_states.state import miss
from lxml import html


class Site(miss.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__

        # If it's the beginning of January, we need to make sure that we aren't
        # missing any late-coming cases from the previous year.
        today = date.today()
        self.url = (
            "http://courts.ms.gov/scripts/websiteX_cgi.exe/GetOpinion?"
            "Year={year}&Court=Court+of+Appeals&Submit=Submit"
        ).format(year=int(today.year) - 1)
        beginning_of_year = (
            date(today.year, 1, 1) <= today <= date(today.year, 1, 15)
        )
        if not beginning_of_year:
            # This simply aborts the crawler.
            self.status = 200
            self.html = html.fromstring("<html></html>")



# Scraper for Florida 4th District Court of Appeal Written Opinions
# CourtID: flaapp4written
# Court Short Name: flaapp4written
# Contact: Errors can be reported to Laura or Brandon, who are the
#  computer people for the court. The phone number is (561) 242-2000

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    """Web Interface: http://www.4dca.org/opinions_auto/most_recent_written.shtml"""

    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.set_url("o")

    def set_url(self, type):
        self.url = "https://edca.4dca.org/opinions.aspx?type=%s" % type

    def _get_docket_numbers(self):
        return self.return_text_for_cell(1)

    def _get_case_names(self):
        return self.return_text_for_cell(3)

    def _get_dispositions(self):
        return self.return_text_for_cell(5)

    def _get_download_urls(self):
        path = "%s/a/@href" % self.cell_path(2)
        return [href for href in self.html.xpath(path)]

    def _get_case_dates(self):
        return [
            convert_date_string(date_string)
            for date_string in self.return_text_for_cell(6)
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def return_text_for_cell(self, cell_number):
        return [
            cell.text_content().strip()
            for cell in self.html.xpath(self.cell_path(cell_number))
        ]

    def cell_path(self, cell_number):
        return '//table[@id="grdOpinions"]//td[%d]' % cell_number



"""
Scraper for Indiana Supreme Court archive before 6/6/2005
CourtID: ind
Court Short Name: Ind.
Auth: Mike Lissner <mike@freelawproject.org>
Reviewer:
History:
    2014-09-03: Renamed to ind_2005.py by janderse
"""
from juriscraper.OpinionSite import OpinionSite

import time
from datetime import date
from lxml import html


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www.in.gov/judiciary/opinions/previous/archsup.html"
        self.court_id = self.__module__

    def _get_case_names(self):
        return [e for e in self.html.xpath("//table/tr/td[1]/font/a/text()")]

    def _get_case_dates(self):
        dates = []
        for link_string in self.html.xpath("//table/tr/td[4]/font/a/@href"):
            date_string = link_string.split("/")[-1].split(".")[0][:-2]
            if date_string == "12319701-fs":
                # special case (there's always *one*)
                dates.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime("123197", "%m%d%y"))
                    )
                )
            elif len(date_string) == 4:
                dates.append(
                    date.fromtimestamp(
                        time.mktime(
                            time.strptime(date_string + "98", "%m%d%y")
                        )
                    )
                )
            elif len(date_string) == 5:
                dates.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime("0" + date_string, "%m%d%y"))
                    )
                )
            elif len(date_string) == 6:
                dates.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime(date_string, "%m%d%y"))
                    )
                )
            else:
                print(date_string)
        return dates

    def _get_download_urls(self):
        return [s for s in self.html.xpath("//table/tr/td[4]/font/a/@href")]

    def _get_docket_numbers(self):
        return [
            html.tostring(e, method="text")
            for e in self.html.xpath("//table/tr/td[2]")
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)



"""Scraper for U.S. Merit Systems Protection Board
CourtID: mspb
Court Short Name: MSPB
Author: Jon Andersen
Reviewer: mlr
Date created: 1 Sep 2014
Type: Precedential
"""

import random
from datetime import datetime

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.column_diff = 0
        # Something fishy is going on with this court's site. They
        # seem to be blocking/hiding results based on our regular
        # 'juriscraper' User-Agent header. We contacted the
        # court multiple times to work work with them on exploring
        # and resolving this issue, but they were virtually worthless
        # in that email exchange. Since they have been unhelpful, cannot
        # provide more information, and don't seem to care, we will
        # utilize this workaround and sacrifice the courtesy of
        # identifying ourselves with the 'juriscraper' UA. The below
        # workaround is required to shows results for this scraper,
        # and also to show results (and prevent timeout) for mspb_u child.
        self.request["headers"] = {
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36"
        }
        self.type = "Precedential"
        self.display = 2368396
        self.set_url()

    def set_url(self):
        cache_key = "a" + str(random.randrange(1, 100000000))
        pattern = (
            "https://www.mspb.gov/MSPBSEARCH/decisiondisplay_2011.aspx?timelapse=12&displaytype=%d&description=%s+Decisions&cachename="
            + cache_key
        )
        self.url = pattern % (self.display, self.type)

    def _get_download_urls(self):
        """Example: http://www.mspb.gov/netsearch/viewdocs.aspx?docnumber=1075720&version=1080035&application=ACROBAT"""
        path = "//tr[@class='ITEMS']/td[%s]/a/@href" % (3 + self.column_diff)
        return list(self.html.xpath(path))

    def _get_case_names(self):
        """Example: {Appellant} v. {Agency}"""
        path = "//tr[@class='ITEMS']/td[%s]/a/text()" % (3 + self.column_diff)
        appellants = self.html.xpath(path)
        path = "//tr[@class='ITEMS']/td[%s]/text()" % (4 + self.column_diff)
        agencies = self.html.xpath(path)
        return [
            "%s v. %s" % (appellant, agency)
            for (appellant, agency) in zip(appellants, agencies)
        ]

    def _get_case_dates(self):
        """Example: Aug 06, 2014"""
        path = "//tr[@class='ITEMS']/td[1]/span/text()"
        return [
            datetime.strptime(date_string, "%b %d, %Y").date()
            for date_string in self.html.xpath(path)
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _get_neutral_citations(self):
        """Example: 2014 MSPB 68"""
        path = "//tr[@class='ITEMS']/td[2]/text()"
        return [s.replace("\u00A0", " ") for s in self.html.xpath(path)]

    def _get_case_name_shorts(self):
        # We don't (yet) support short case names for administrative bodies.
        return None



from collections import defaultdict
import os
import signal
import six
import sys
import traceback
from six.moves.urllib import parse as six_parse
from six.moves.urllib import request as six_request
from optparse import OptionParser

from juriscraper.lib.importer import build_module_list, site_yielder
from juriscraper.lib.string_utils import trunc
from juriscraper.report import generate_scraper_report

die_now = False


def signal_handler(signal, frame):
    # Trigger this with CTRL+4
    v_print(3, "**************")
    v_print(3, "Signal caught. Finishing the current court, then exiting...")
    v_print(3, "**************")
    global die_now
    die_now = True


def extract_doc_content(data):
    # Your data extraction routines here. (pdftotext, abiword, etc.)
    return data


def scrape_court(site, binaries=False):
    """Calls the requested court(s), gets its content, then throws it away.

    Note that this is a very basic caller lacking important functionality, such
    as:
     - checking whether the HTML of the page has changed since last visited
     - checking whether the downloaded content is already in your data store
     - saving anything at all

    Nonetheless, this caller is useful for testing, and for demonstrating some
    basic pitfalls that a caller will run into.
    """
    exceptions = defaultdict(list)
    for item in site:
        # First turn the download urls into a utf-8 byte string
        item_download_urls = item["download_urls"].encode("utf-8")
        # Percent encode URLs (this is a Python wart)
        download_url = six_parse.quote(
            item_download_urls, safe="%/:=&?~#+!$,;'@()*[]"
        )

        if binaries:
            try:
                opener = six_request.build_opener()
                for cookie_dict in site.cookies:
                    opener.addheaders.append(
                        (
                            "Cookie",
                            "%s=%s"
                            % (cookie_dict["name"], cookie_dict["value"]),
                        )
                    )
                data = opener.open(download_url).read()
                # test for empty files (thank you CA1)
                if len(data) == 0:
                    exceptions["EmptyFileError"].append(download_url)
                    v_print(3, "EmptyFileError: %s" % download_url)
                    v_print(3, traceback.format_exc())
                    continue
            except Exception:
                exceptions["DownloadingError"].append(download_url)
                v_print(3, "DownloadingError: %s" % download_url)
                v_print(3, traceback.format_exc())
                continue

            # Extract the data using e.g. antiword, pdftotext, etc., then
            # clean it up.
            data = extract_doc_content(data)
            data = site.cleanup_content(data)

        # Normally, you'd do your save routines here...
        v_print(1, "\nAdding new item:")
        for k, v in item.items():
            if isinstance(v, six.text_type):
                value = trunc(v, 200, ellipsis="...")
                v_print(1, '    %s: "%s"' % (k, value.encode("utf-8")))
            else:
                # Dates and such...
                v_print(1, "    %s: %s" % (k, v))

    v_print(
        3, "\n%s: Successfully crawled %d items." % (site.court_id, len(site))
    )
    return {"count": len(site), "exceptions": exceptions}


v_print = None


def main():
    global die_now

    # this line is used for handling SIGTERM (CTRL+4), so things can die safely
    signal.signal(signal.SIGTERM, signal_handler)

    usage = (
        "usage: %prog -c COURTID [-d|--daemon] [-b|--binaries] [-r|--report]\n\n"
        "To test ca1, downloading binaries, use: \n"
        "    python %prog -c opinions.united_states.federal_appellate.ca1 -b\n\n"
        "To test all federal courts, omitting binaries, use: \n"
        "    python %prog -c opinions.united_states.federal_appellate"
        "Passing the --report option will generate an HTML report in "
        "the root directory after scrapers have run"
    )
    parser = OptionParser(usage)
    parser.add_option(
        "-c",
        "--courts",
        dest="court_id",
        metavar="COURTID",
        help=(
            "The court(s) to scrape and extract. This should be in "
            "the form of a python module or package import "
            "from the Juriscraper library, e.g. "
            '"juriscraper.opinions.united_states.federal.ca1" or '
            'simply "opinions" to do all opinions. If desired, '
            "you can use slashes instead of dots to separate"
            "the import path."
        ),
    )
    parser.add_option(
        "-d",
        "--daemon",
        action="store_true",
        dest="daemonmode",
        default=False,
        help=(
            "Use this flag to turn on daemon "
            "mode, in which all courts requested "
            "will be scraped in turn, non-stop."
        ),
    )
    parser.add_option(
        "-b",
        "--download_binaries",
        action="store_true",
        dest="binaries",
        default=False,
        help=(
            "Use this flag if you wish to download the pdf, "
            "wpd, and doc files."
        ),
    )
    parser.add_option(
        "-v",
        "--verbosity",
        action="count",
        default=1,
        help="Increase output verbosity (e.g., -vv is more than -v).",
    )
    parser.add_option(
        "--backscrape",
        dest="backscrape",
        action="store_true",
        default=False,
        help="Download the historical corpus using the _download_backwards method.",
    )
    parser.add_option(
        "-r",
        "--report",
        action="store_true",
        default=False,
        help="Generate a report.html with the outcome of running the scrapers",
    )

    (options, args) = parser.parse_args()

    daemon_mode = options.daemonmode
    binaries = options.binaries
    court_id = options.court_id
    backscrape = options.backscrape
    generate_report = options.report

    # Set up the print function
    print("Verbosity is set to: %s" % options.verbosity)

    def _v_print(*verb_args):
        if verb_args[0] > (3 - options.verbosity):
            print(verb_args[1])

    global v_print
    v_print = _v_print

    results = {}

    if not court_id:
        parser.error("You must specify a court as a package or module.")
    else:
        court_id = court_id.replace("/", ".")
        if court_id.endswith(".py"):
            court_id = court_id[:-3]

        module_strings = build_module_list(court_id)
        if len(module_strings) == 0:
            parser.error("Unable to import module or package. Aborting.")

        v_print(3, "Starting up the scraper.")
        num_courts = len(module_strings)
        i = 0
        while i < num_courts:
            current_court = module_strings[i]
            results[current_court] = {"global_failure": False}
            # this catches SIGINT, so the code can be killed safely.
            if die_now:
                v_print(3, "The scraper has stopped.")
                sys.exit(1)

            package, module = module_strings[i].rsplit(".", 1)
            v_print(3, "Current court: %s.%s" % (package, module))

            mod = __import__(
                "%s.%s" % (package, module), globals(), locals(), [module]
            )
            try:
                if backscrape:
                    for site in site_yielder(
                        mod.Site().back_scrape_iterable, mod
                    ):
                        site.parse()
                        scrape_court(site, binaries)
                else:
                    site = mod.Site()
                    v_print(
                        3, "Sent %s request to: %s" % (site.method, site.url)
                    )
                    if site.uses_selenium:
                        v_print(3, "Selenium will be used.")
                    site.parse()
                    results[current_court]["scrape"] = scrape_court(
                        site, binaries
                    )
            except Exception:
                results[current_court][
                    "global_failure"
                ] = traceback.format_exc()
                results[current_court]["scrape"] = {}
                v_print(3, "*************!! CRAWLER DOWN !!****************")
                v_print(
                    3,
                    "*****scrape_court method failed on mod: %s*****"
                    % module_strings[i],
                )
                v_print(3, "*************!! ACTION NEEDED !!***************")
                v_print(3, traceback.format_exc())
                i += 1
                continue

            last_court_in_list = i == (num_courts - 1)
            if last_court_in_list and daemon_mode:
                i = 0
            else:
                i += 1

    v_print(3, "The scraper has stopped.")

    if generate_report:
        report_path = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "../report.html")
        )
        v_print(3, "Generating HTML report at %s" % report_path)
        generate_scraper_report(report_path, results)

    sys.exit(0)


if __name__ == "__main__":
    main()



# coding=utf-8
"""Scraper for Colorado Supreme Court
CourtID: colo
Court Short Name: Colo.
Author: Philip Ardery
Reviewer: mlr
Date created: 2016-06-03
Contact: Email "Internet and Technology" staff listed at http://www.cobar.org/staff
         they usually fix issues without resonding to the emails directly. You can
         also try submitting the form here: http://www.cobar.org/contact
"""

import re
from lxml import html
from urlparse import urlparse

from juriscraper.AbstractSite import logger
from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.exceptions import InsanityException
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    sub_page_opinion_link_path = "//a[@class='Head articletitle']"
    parent_summary_block_path = (
        'parent::p/following-sibling::div[@class="Normal"][1]'
    )

    # I'm certain this can be done more professionally,
    # but I (arderyp) am not gifted at the art of regex
    regex = "(?:No.\s)?(\d+)\s+(\w+)(?:\s+)?(\d+M?\.?)\s*((Nos?\.?\s+)?((\w{5,8}\.?)(((\s+\&|\,)\s+\w{5,8})+)?))\.?(\s+)?(.*)"

    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.cobar.org/For-Members/Opinions-Rules-Statutes/Colorado-Supreme-Court-Opinions"
        self.base_path = "//div[@id='dnn_ctr2509_ModuleContent']/ul/li/a"
        self.next_subpage_path = "//a[@id='dnn_ctr2512_DNNArticle_List_MyArticleList_MyPageNav_cmdNext']"
        self.cases = []

    def _download(self, request_dict={}):
        self.request_dict = request_dict
        landing_page_html = super(Site, self)._download(request_dict)

        # Test/example files should use html from direct resource page
        #
        # PLEASE NOTE: if your adding a new example file, ensure that,
        # if any of the opinion links on the page do not link directly to
        # a pdf url, that you manually edit your example file and add '.pdf'
        # to the end of all those opinion anchor hrefs. We do this in order
        # to prevent the tests form hitting the network.  HINT: if your new
        # test takes any more than a split second to run, its probably hitting
        # the network and needs ot be fixed as explained above.
        #
        # PLEASE ALSO NOTE: coloctapp_example_3.html is supposed to have 0
        # results.  It is a blank page test case covering is_this_a_blank_page().
        if self.test_mode_enabled():
            date_string = landing_page_html.xpath("//h3")[0].text_content()
            date_obj = convert_date_string(date_string)
            self._extract_cases_from_sub_page(landing_page_html, date_obj)
            return [landing_page_html]

        html_trees = []
        html_list = [landing_page_html]

        while len(html_list) > 0:
            html_l = html_list[0]
            html_list = html_list[1:]

            # Loop over sub-pages
            hrefs = html_l.xpath(self.base_path)
            for ahref in hrefs:
                date_string = ahref.xpath("./text()")[0]
                url = ahref.xpath("./@href")[0]
                date_obj = convert_date_string(date_string)
                logger.info("Getting sub-url: %s" % url)

                # Fetch sub-page's content
                html_tree = self._get_html_tree_by_url(url, self.request_dict)
                self._extract_cases_from_sub_page(html_tree, date_obj)
                html_trees.append((html_tree, date_obj))

                # DEACTIVATED BY arderyp ON 2018.06.07, SEE NOTE ON get_next_page()
                # # process all sub-pages
                # if self.next_subpage_path and self.test_mode_enabled():
                #     while True:
                #         next_subpage_html = self.get_next_page(html_tree, self.next_subpage_path, request_dict, url)
                #         if next_subpage_html is None:
                #             break
                #
                #         self._extract_cases_from_sub_page(next_subpage_html, date_obj)
                #         html_trees.append((next_subpage_html, date_obj))
                #         html_tree = next_subpage_html

        return html_trees

    # CALLS TO THIS FUNCTION DEACTIVATED BY arderyp ON 2018.06.07
    # This is (temporarily) disabled because it no longer works,
    # and I (arderyp) am not familiar enough with asp.net to get
    # it working agian without investing a lot of time.  In short,
    # this process is finding the "Next" pagination button, collecting
    # the asp.net <input> field/values from the page, and posting
    # these inputs back to the same page, using the special "Next"
    # button hash/id value.  However, something isn't working, because
    # the posted fields are not being picked up properly, as the same
    # page loads again, instead of the next page.  As a result, we
    # enter an infinite look of finding the next button, submitting
    # the data, landing back on the same page, finding the samenext
    # button, submitting the data, landing back on the same page yet
    # again... end on and on and on. Example pages to test againast:
    #   http://www.cobar.org/For-Members/Opinions-Rules-Statutes/Colorado-Court-of-Appeals-Opinions/Date/bdate/2018-4-19/edate/2018-4-19/cid/6
    #   http://www.cobar.org/For-Members/Opinions-Rules-Statutes/Colorado-Supreme-Court-Opinions/Date/bdate/2017-6-5/edate/2017-6-5/cid/4
    #
    def get_next_page(self, html_l, next_page_xpath, request_dict, url):
        result = None
        for nhref in html_l.xpath(next_page_xpath):
            evtarget = None
            class_, href = nhref.get("class"), nhref.get("href")
            exp_start, exp_end = "javascript:__doPostBack('", "','')"
            if (
                href is not None
                and href.startswith(exp_start)
                and href.endswith(exp_end)
            ):
                evtarget = href[len(exp_start) : -len(exp_end)]
            # a disabled 'next page' link will contains this string in the class list
            if evtarget is not None and "aspNetDisabled" not in class_.split():
                form = [("__EVENTTARGET", ("", evtarget))]
                form_items_xpath = "//input"
                for form_el in html_l.xpath(form_items_xpath):
                    name = form_el.get("name")
                    if name is not None:
                        value = form_el.get("value") or ""
                        # 1st tuple item is empty: don't include filename part
                        # in the multipart/form-data request
                        form.append((name, ("", value)))
                # use 'files' arg to make the request multipart/form-data
                files_request_dict = dict(request_dict)
                files_request_dict["files"] = form
                # temporarily override method and url
                prev_method, prev_url = self.method, self.url
                self.method = "POST"
                self.url = url
                # _download() in AbstractSite.py expects this to be not None
                self.parameters = {}
                result = super(Site, self)._download(files_request_dict)
                # restore method and url
                self.method = prev_method
                self.url = prev_url
        return result

    def _extract_cases_from_sub_page(self, html_tree, date_obj):
        anchors = html_tree.xpath("//a[@class='Head articletitle']")
        if self.is_this_a_blank_page(anchors, date_obj):
            return []
        for anchor in anchors:
            text = self._extract_text_from_anchor(anchor)
            if self.is_this_skippable_date_anchor(text):
                # Sometimes COBAR includes link to worthless Opinion
                # Announcement PDF provided by colo court site. These
                # links usually have text that is a simple date string.
                # Skip them.
                continue
            name = self._extract_name_from_text(text)
            url = self._extract_url_from_anchor(anchor)

            # For whatever insane reason, some pages provide direct links
            # to PDFs, and other link off to subpages that must themselves
            # be scraped to extract the real PDF url.
            if not self.is_url_pdf(url):
                (
                    alt_url,
                    alt_name,
                ) = self.extract_pdf_url_and_name_from_resource(url)
                url = alt_url if alt_url else url
                name = alt_name if alt_name else name

            self.cases.append(
                {
                    "url": url,
                    "name": name,
                    "date": date_obj,
                    "status": "Published",
                    "docket": self._extract_docket_from_text(text),
                    "citation": self._extract_citation_from_text(text),
                    "summary": self._extract_summary_relative_to_anchor(
                        anchor
                    ),
                    "nature": self._extract_nature_relative_to_anchor(anchor),
                }
            )

    @classmethod
    def _extract_docket_from_text(cls, text):
        text = text.strip()
        try:
            match = re.match(cls.regex, text).group(6)
        except:
            raise InsanityException('Unable to parse docket from "%s"' % text)
        dockets_raw = match.rstrip(".").replace("&", " ").replace(",", " ")
        dockets = dockets_raw.split()
        return ", ".join(dockets)

    @classmethod
    def _extract_name_from_text(cls, text):
        text = text.strip()
        try:
            match = re.match(cls.regex, text).group(12)
        except:
            raise InsanityException(
                'Unable to parse case name from "%s"' % text
            )
        return match.strip().rstrip(".")

    @classmethod
    def _extract_citation_from_text(cls, text):
        return text.lstrip("No.").split(".")[0].strip()

    @classmethod
    def _extract_text_from_anchor(cls, anchor):
        text = anchor.xpath("text()")[0]
        text = text.replace("Nos.", "No.")
        if "No." in text and "No. " not in text:
            text = text.replace("No.", "No. ")
        return text

    @classmethod
    def _extract_url_from_anchor(cls, anchor):
        return anchor.xpath("@href")[0]

    def extract_pdf_url_and_name_from_resource(self, url):
        """Scrape the resource for a direct link to PDF or a backup case name

        A resource can only have one or the other.  If the PDF is embedded,
        the name is not displayed in plain text format. These return values
        are used as backups in case the original source data is non-ideal.
        """
        html_tree = self._get_html_tree_by_url(url, self.request_dict)
        href = html_tree.xpath("//h2/a[contains(@href, '.pdf')]/@href")
        if href:
            return href[0], False
        path1 = '//p/strong[starts-with(text(), "Plaintiff-Appellee:")]/parent::p/text()'
        path2 = (
            '//p/strong[starts-with(text(), "Petitioner:")]/parent::p/text()'
        )
        parts = html_tree.xpath("%s | %s" % (path1, path2))
        if parts:
            name = " ".join(part.strip() for part in parts if part.strip())
            return False, name
        return False, False

    def _extract_summary_relative_to_anchor(self, anchor):
        parts = anchor.xpath("%s/p" % self.parent_summary_block_path)
        return " ".join([part.text_content().strip() for part in parts])

    def _extract_nature_relative_to_anchor(self, anchor):
        """Extract italicized nature summary that appears directly after download url

        The court uses a lot of different html method of presenting this information.
        If a "nature" field is showing blank in the results, it could be that they are
        using a new html path, which should be added to the path_patterns list below.
        """
        nature = ""

        # The order of this list matters.  Generally, put
        # the more complex paths as the top
        path_patterns = [
            "%s/p/strong/em/span/text()",
            "%s/p/em/text()",
            "%s/p/em/span/text()",
            "%s/p/i/span/text()",
            "%s/p/i/text()",
            "%s/em/text()",
            "%s/em/span/text()",
            "%s/p/span/i/text()",
            "%s/span/em/text()",
            "%s/p/strong/em/text()",
            "%s/strong/em/text()",
            "%s/span/text()",
            "%s/p/span/em/text()",
        ]
        for pattern in path_patterns:
            try:
                nature = anchor.xpath(
                    pattern % self.parent_summary_block_path
                )[0]
                break
            except:
                continue
        return nature.strip().rstrip(".")

    @staticmethod
    def cleanup_content(content):
        tree = html.fromstring(content)
        core_element = tree.xpath('//div[@class="contentmain"]')
        if core_element:
            # its an HTML opinion, return only the core element
            return html.tostring(
                core_element[0], pretty_print=True, encoding="unicode"
            )
        else:
            # its raw pdf file content, which doesn't need to be manipulated
            return content

    def is_this_a_blank_page(self, anchors, date_obj):
        """Man, colo/COBAR are not mkaing things easy for us.
        This is an unfortunate workaround for a recurring problem
        where the court publishes blank pages, which they don't
        populate with data until days/weeks later. Return True
        if there are no anchors, or there is only one anchor
        whose text is just the date string, a thing they do
        often for whatever reason.
        """
        anchor_count = len(anchors)
        if not anchor_count:
            return True
        if anchor_count == 1:
            text = self._extract_text_from_anchor(anchors[0])
            return self.is_this_skippable_date_anchor(text)

    def is_this_skippable_date_anchor(self, text):
        """Return true is link text is parsible date"""
        try:
            convert_date_string(text)
            return True
        except:
            pass
        return False

    def is_url_pdf(self, url):
        parsed = urlparse(url)
        return parsed.path.lower().endswith(".pdf")

    def _get_case_names(self):
        return [case["name"] for case in self.cases]

    def _get_download_urls(self):
        return [case["url"] for case in self.cases]

    def _get_case_dates(self):
        return [case["date"] for case in self.cases]

    def _get_docket_numbers(self):
        return [case["docket"] for case in self.cases]

    def _get_neutral_citations(self):
        return [case["citation"] for case in self.cases]

    def _get_precedential_statuses(self):
        return [case["status"] for case in self.cases]

    def _get_summaries(self):
        return [case["summary"] for case in self.cases]

    def _get_nature_of_suit(self):
        return [case["nature"] for case in self.cases]



from juriscraper.lib.html_utils import get_html5_parsed_text
from juriscraper.opinions.united_states.federal_appellate import ca11_p


class Site(ca11_p.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://media.ca11.uscourts.gov/opinions/unpub/logname.php"
        self.court_id = self.__module__
        self.back_scrape_iterable = range(20, 22000, 20)

    def _make_html_tree(self, text):
        return get_html5_parsed_text(text)

    def _download_backwards(self, n):
        self.url = "http://media.ca11.uscourts.gov/opinions/unpub/logname.php?begin={}&num={}&numBegin=1".format(
            n, n / 20 - 1
        )

        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200



"""Scraper for First Circuit of Appeals
CourtID: ca1
Court Short Name: ca1
Author: Michael Lissner
Date created: 13 June 2014
"""

import re
from juriscraper.OralArgumentSite import OralArgumentSite
from juriscraper.lib.string_utils import clean_if_py3
from juriscraper.lib.string_utils import convert_date_string


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://media.ca1.uscourts.gov/files/audio/audiorss.php"

    def _get_download_urls(self):
        # For some reason, lxml is being weird while parsing this XML and is
        # munging the link node into a text node. This xpath seems to work
        # despite it being rather wonky.
        path = "//item/link"
        download_urls = []
        for t in self.html.xpath(path):
            download_urls.append(t.tail)
        return download_urls

    def _get_case_names(self):
        case_names = []
        for t in self.html.xpath("//item/title/text()"):
            case_name = t.split(", ", 1)[1]
            case_names.append(case_name)
        return case_names

    def _get_case_dates(self):
        path = "//item/description/b/text()"
        dates = []
        for t in self.html.xpath(path):
            # t looks like: [Argued:91-1-2015]
            t = re.sub(r"[\[\]\s]", "", t)  # Strip out [ and ].
            date_string = clean_if_py3(t).split(":", 1)[1].strip()
            # sometimes there is a type like: [Argued:91-1-2015mp3]
            # such as in ca1_example_2.xml
            date_string = date_string.replace("mp3", "")
            dates.append(convert_date_string(date_string))
        return dates

    def _get_docket_numbers(self):
        document_numbers = []
        for t in self.html.xpath("//item/title/text()"):
            case_name = t.split(", ", 1)[0]
            case_name = re.sub(r"case:\s?", "", case_name, re.I)
            document_numbers.append(case_name)
        return document_numbers



"""
Scraper for New York Court of Appeals
CourtID: ny
Court Short Name: NY
History:
 2014-07-04: Created by Andrei Chelaru, reviewed by mlr.
 2015-10-23: Parts rewritten by mlr.
 2016-05-04: Updated by arderyp to handle typos in docket string format
"""

import re
from datetime import date

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.html_utils import get_html5_parsed_text
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    DOWNLOAD_URL_SUB_PATH = "td[2]//@href[not(contains(., 'DecisionList'))]"
    FOUR_CELLS_SUB_PATH = "//*[count(td)=3"

    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        today = date.today()
        # https://www.nycourts.gov/ctapps/Decisions/2015/Dec15/December15.html
        self.url = "http://www.nycourts.gov/ctapps/Decisions/{year}/{mon}{yr}/{month}{yr}.html".format(
            year=today.year,
            yr=today.strftime("%y"),
            mon=today.strftime("%b"),
            month=today.strftime("%B"),
        )
        self.court_id = self.__module__

    def _make_html_tree(self, text):
        return get_html5_parsed_text(text)

    def _get_case_names(self):
        path = "%s and %s]" % (
            self.FOUR_CELLS_SUB_PATH,
            self.DOWNLOAD_URL_SUB_PATH,
        )
        case_names = []
        for element in self.html.xpath(path):
            case_name_parts = []
            for t in element.xpath("./td[3]/p/font/text()"):
                if t.strip():
                    case_name_parts.append(t)
            if not case_name_parts:
                # No hits for first XPath, try another that sometimes works.
                for t in element.xpath("./td[3]//text()"):
                    if t.strip():
                        case_name_parts.append(t)
            if case_name_parts:
                case_names.append(", ".join(case_name_parts))
        return case_names

    def _get_download_urls(self):
        return self.html.xpath(
            "%s]/%s" % (self.FOUR_CELLS_SUB_PATH, self.DOWNLOAD_URL_SUB_PATH)
        )

    def _get_case_dates(self):
        case_dates = []
        case_date = False
        # Process rows. If it's a date row,
        # save the date, continue, then add
        # date for each opinion row below it
        for row in self.html.xpath("//tr[not(.//table)]"):
            date_from_row = self.get_date_from_text(row.text_content())
            if date_from_row:
                case_date = date_from_row
                continue
            elif self._row_contains_opinion(row) and case_date:
                case_dates.append(case_date)
        return case_dates

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_docket_numbers(self):
        docket_numbers = []
        for cell in self.html.xpath("%s]/td[1]" % self.FOUR_CELLS_SUB_PATH):
            text = cell.text_content()
            date_from_text = self.get_date_from_text(text)
            if not date_from_text:
                if re.search(r"N(o|O|0)\.?,?", text):
                    docket = self._sanitize_docket_string(text)
                    docket_numbers.append(docket)
        return docket_numbers

    def _sanitize_docket_string(self, raw_docket_string):
        """Handle typos and non-standard docket number strings

        Dockets on this page should be in format of "No. #",
        but sometimes they forget the period, or use a comma
        instead.  We want to trip all variations of that out
        and replace slash delimiters with coma delimiters.
        """
        for abbreviation in ["No.", "No ", "No, ", "NO. "]:
            raw_docket_string = raw_docket_string.replace(abbreviation, "")
        return ", ".join(raw_docket_string.split(" / "))

    def _row_contains_opinion(self, row):
        p1 = "./td[3]"
        p2 = "./%s" % self.DOWNLOAD_URL_SUB_PATH
        return row.xpath(p1) and row.xpath(p2)

    def get_date_from_text(self, text):
        try:
            return convert_date_string(text)
        except ValueError:
            return False



"""Scraper for the Air Force Court of Criminal Appeals
CourtID: afcca
Court Short Name: Air Force Court of Criminal Appeals
Reviewer: mlr
History:
    15 Sep 2014: Created by Jon Andersen
"""

from datetime import date

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.back_scrape_iterable = range(2013, 2002 - 1, -1)
        self.url_base = "http://afcca.law.af.mil/content/opinions_date_%d.html"
        self.url = self.url_base % date.today().year
        self.table_cell_path = '//table[@width="800"][@align="center"]/tr[position() > 2]/td[%d]%s'

        # HTTPS certificate is bad, but hopefully they'll fix it and we can remove the line below
        self.disable_certificate_verification()

    def cell_path(self, cell, substring=""):
        return self.table_cell_path % (cell, substring)

    def _get_case_dates(self):
        path = self.cell_path(5)
        return [
            convert_date_string(cell.text_content())
            for cell in self.html.xpath(path)
        ]

    def _get_case_names(self):
        path = self.cell_path(2, "/a")
        return [cell.text_content() for cell in self.html.xpath(path)]

    def _get_download_urls(self):
        path = self.cell_path(2, "/a/@href")
        return [href for href in self.html.xpath(path)]

    def _get_docket_numbers(self):
        path = self.cell_path(3)
        return [
            "ACM %s" % cell.text_content() for cell in self.html.xpath(path)
        ]

    def _get_precedential_statuses(self):
        statuses = []
        path = self.cell_path(4)
        for cell in self.html.xpath(path):
            text = cell.text_content()
            statuses.append(
                "Published" if "Published" in text else "Unpublished"
            )
        return statuses

    def _download_backwards(self, year):
        self.url = self.url_base % year
        self.html = self._download()



# Author: Michael Lissner
# Date created: 2013-06-06

import re
import requests

from datetime import date
from datetime import datetime

from juriscraper.opinions.united_states.state import nd
from juriscraper.DeferringList import DeferringList


class Site(nd.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        today = date.today()
        self.url = "http://www.ndcourts.gov/opinions/month/%s.htm" % (
            today.strftime("%b%Y")
        )

    def _get_download_urls(self):
        """We use a fetcher and a DeferringList object and a HEAD request
        to test whether the wpd exists for a case"""

        def fetcher(html_link):
            if self.test_mode_enabled():
                return html_link  # Can't fetch remote during tests
            case_number = re.search(r"(\d+)", html_link).group(0)
            wpd_link = "http://www.ndcourts.gov/wp/%s.wpd" % case_number
            r = requests.head(
                wpd_link,
                allow_redirects=False,
                headers={"User-Agent": "Juriscraper"},
            )
            if r.status_code == 200:
                return wpd_link
            else:
                return html_link

        if self.crawl_date >= date(1998, 10, 1):
            path = '//a/@href[contains(., "/court/opinions/")]'
            seed = list(self.html.xpath(path))
        else:
            path = "//ul//a[text()]/@href"
            seed = list(self.html.xpath(path))
        return DeferringList(seed=seed, fetcher=fetcher)

    def _get_case_names(self):
        if self.crawl_date >= date(1998, 10, 1):
            path = '//a[contains(@href, "/court/opinions/")]/text()'
            return list(self.html.xpath(path))
        else:
            path = "//ul//a/text()"
            names = self.html.xpath(path)
            case_names = []
            if self.crawl_date < date(1996, 11, 1):
                # A bad time.
                for name in names:
                    name = name.rsplit("-")[0]
                    case_names.append(name)
                return case_names
            else:
                return list(names)

    def _get_case_dates(self):
        # A tricky one. We get the case dates, but each can have different number of cases below it, so we have to
        # count them.
        case_dates = []
        if self.crawl_date >= date(1998, 10, 1):
            test_path = "//body/a"
            if len(self.html.xpath(test_path)) == 0:
                # It's a month with no cases (like Jan, 2009)
                return []
            path = "//body/a|//body/font"
            for e in self.html.xpath(path):
                if e.tag == "font":
                    date_str = e.text
                    dt = datetime.strptime(date_str, "%B %d, %Y").date()
                elif e.tag == "a":
                    try:
                        case_dates.append(dt)
                    except NameError:
                        # When we don't yet have the date
                        continue
        else:
            path = "//h4|//li"
            for e in self.html.xpath(path):
                if e.tag == "h4":
                    # We make dates on h4's because there's one h4 per date.
                    date_str = e.text.strip()
                    dt = datetime.strptime(date_str, "%B %d, %Y").date()
                elif e.tag == "li":
                    try:
                        # We append on li's, because there's one li per case.
                        case_dates.append(dt)
                    except NameError:
                        # When we don't yet have the date
                        continue
        return case_dates

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_docket_numbers(self):
        if self.crawl_date >= date(1998, 10, 1):
            path = '//a/@href[contains(., "/court/opinions/")]'
        else:
            path = "//ul//a[text()]/@href"
        docket_numbers = []
        for html_link in self.html.xpath(path):
            try:
                docket_numbers.append(re.search(r"(\d+)", html_link).group(0))
            except AttributeError:
                continue
        return docket_numbers

    def _get_neutral_citations(self):
        if self.crawl_date < date(1997, 2, 1):
            # Old format, but no neutral cites, thus short circuit the function.
            return None
        elif self.crawl_date < date(1998, 10, 1):
            # Old format with: 1997 ND 30 - Civil No. 960157 or 1997 ND 30
            path = "//li/text()"
        elif self.crawl_date >= date(1998, 10, 1):
            # New format with: 1997 ND 30
            path = "//body/text()"
        neutral_cites = []
        for t in self.html.xpath(path):
            try:
                neutral_cites.append(
                    re.search(
                        r"^.{0,5}(\d{4} ND (?:App )?\d{1,4})", t, re.MULTILINE
                    ).group(1)
                )
            except AttributeError:
                continue
        return neutral_cites

    def _post_parse(self):
        # Remove any information that applies to non-appellate cases.
        if self.neutral_citations:
            delete_items = []
            for i in range(0, len(self.neutral_citations)):
                if "App" in self.neutral_citations[i]:
                    delete_items.append(i)

            for i in sorted(delete_items, reverse=True):
                del self.download_urls[i]
                del self.case_names[i]
                del self.case_dates[i]
                del self.precedential_statuses[i]
                del self.docket_numbers[i]
                del self.neutral_citations[i]
        else:
            # When there aren't any neutral cites that means they're all supreme court cases.
            pass

    def _download_backwards(self, d):
        self.crawl_date = d
        self.url = "http://www.ndcourts.gov/opinions/month/%s.htm" % (
            d.strftime("%b%Y")
        )
        self.html = self._download()



from juriscraper.OpinionSite import OpinionSite
import time
from datetime import date
from lxml import html


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = (
            "http://www.armfor.uscourts.gov/newcaaf/opinions/2005Term.htm"
        )
        self.court_id = self.__module__

    def _get_case_names(self):
        return [t for t in self.html.xpath("//table//tr/td[1]/font/text()")]

    def _get_download_urls(self):
        return [
            t
            for t in self.html.xpath(
                '//table//tr/td[2]/font/a/@href[contains(., ".pdf")]'
            )
        ]

    def _get_case_dates(self):
        dates = []
        for e in self.html.xpath(
            "//table/tr[3]/td[2]/blockquote/table/tbody/tr[position() > 1]/td[3]"
        ):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            dates.append(
                date.fromtimestamp(time.mktime(time.strptime(s, "%b %d, %Y")))
            )
        return dates

    def _get_docket_numbers(self):
        docket_numbers = []
        for e in self.html.xpath("//table//table//tr[position() > 1]/td[2]"):
            s = html.tostring(e, method="text", encoding="unicode")
            docket_numbers.append(s.strip()[:-5])
        return docket_numbers

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_neutral_citations(self):
        neutral_citations = []
        for e in self.html.xpath(
            "//table/tr[3]/td[2]/blockquote/table/tbody/tr[position() > 1]/td[4]"
        ):
            s = html.tostring(e, method="text", encoding="unicode")
            neutral_citations.append(s)
        return neutral_citations



"""Scraper for Supreme Court of Maine
CourtID: me
Court Short Name: Me.
Author: Brian W. Carver
Date created: June 20, 2014

History:
  2014-06-25 (est): Added code for additional date formats.
  2014-07-02: Was receiving InsanityException and tweaked date code to get some
              missing dates.
  2014-12-15: Fixes insanity exception by tweaking the XPaths.
"""

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string
from lxml import html


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.courts.maine.gov/opinions_orders/supreme/publishedopinions.shtml"

    def _get_download_urls(self):
        path = "//table//tr/td[2]/a[1]/@href"
        return list(self.html.xpath(path))

    def _get_case_names(self):
        case_names = []
        for e in self.html.xpath("//table//tr/td[2]/a[1]"):
            s = html.tostring(e, method="text", encoding="unicode")
            case_names.append(s)
        return case_names

    def _get_case_dates(self):
        path = '//table[contains(.//th[1], "Opinion")]//tr/td[3]'
        dates = []
        for cell in self.html.xpath(path):
            date_string = cell.text_content().replace("Aguust", "August")
            dates.append(convert_date_string(date_string))
        return dates

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_neutral_citations(self):
        path = "//table[position() > 1]//tr/td[1]//text()"
        return list(self.html.xpath(path))



# -*- coding: utf-8 -*-
"""
Python 2.x Regular Express patterns for string_utils.py
"""
import re

BIG = (
    "3D|AFL|AKA|A/K/A|BMG|CBS|CDC|CDT|CEO|CIO|CNMI|D/B/A|DOJ|DVA|EFF|FCC|"
    "FTC|HSBC|IBM|II|III|IV|JJ|LLC|LLP|MCI|MJL|MSPB|ND|NLRB|PTO|SD|UPS|RSS|SEC|UMG|US|USA|USC|"
    "USPS|WTO"
)
SMALL = u"a|an|and|as|at|but|by|en|for|if|in|is|of|on|or|the|to|v\.?|via|vs\.?"
NUMS = u"0123456789"
PUNCT = ur"""!"#$¢%&'‘()*+,\-./:;?@[\\\]_—`{|}~"""
WEIRD_CHARS = ur"¼½¾§ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÑÒÓÔÕÖØÙÚÛÜßàáâãäåæçèéêëìíîïñòóôœõöøùúûüÿ"
BIG_WORDS = re.compile(ur"^(%s)[%s]?$" % (BIG, PUNCT), re.I | re.U)
SMALL_WORDS = re.compile(r"^(%s)$" % SMALL, re.I)
SMALL_WORD_INLINE = re.compile(r"(^|\s)(%s)(\s|$)" % SMALL, re.I | re.U)
INLINE_PERIOD = re.compile(r"[a-z][.][a-z]", re.I)
INLINE_SLASH = re.compile(r"[a-z][/][a-z]", re.I)
INLINE_AMPERSAND = re.compile(r"([a-z][&][a-z])(.*)", re.I)
UC_ELSEWHERE = re.compile(ur"[%s]*?[a-zA-Z]+[A-Z]+?" % PUNCT, re.U)
CAPFIRST = re.compile(ur"^[%s]*?([A-Za-z])" % PUNCT)
SMALL_FIRST = re.compile(ur"^([%s]*)(%s)\b" % (PUNCT, SMALL), re.I | re.U)
SMALL_LAST = re.compile(r"\b(%s)[%s]?$" % (SMALL, PUNCT), re.I | re.U)
SUBPHRASE = re.compile(r"([:;?!][ ])(%s)" % SMALL)
APOS_SECOND = re.compile(r"^[dol]{1}['‘]{1}[a-z]+$", re.I)
ALL_CAPS = re.compile(ur"^[A-Z\s%s%s%s]+$" % (PUNCT, WEIRD_CHARS, NUMS))
UC_INITIALS = re.compile(r"^(?:[A-Z]{1}\.{1}|[A-Z]{1}\.{1}[A-Z]{1})+,?$")
MAC_MC = re.compile(r"^([Mm]a?c)(\w+.*)")



"""Backscraper for Supreme Court of Maine 2013
CourtID: me
Court Short Name: Me.
Author: Brian W. Carver
Date created: June 20, 2014
"""

from datetime import date
from datetime import datetime
from lxml import html

from juriscraper.OpinionSite import OpinionSite
from juriscraper.opinions.united_states.state import me


class Site(me.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.courts.maine.gov/opinions_orders/supreme/lawcourt/2013/2013_index.shtml"



__all__ = [
    #'ala', Dead. Murdered by terrible site and per-page download fees.
    "alaska",
    "alaskactapp",
    "ariz",
    "arizctapp_div_1",
    "arizctapp_div_2",
    "ark",
    "arkctapp",
    "cal",
    "calag",
    "calctapp_1st",
    "calctapp_2nd",
    "calctapp_3rd",
    "calctapp_4th_div1",
    "calctapp_4th_div2",
    "calctapp_4th_div3",
    "calctapp_5th",
    "calctapp_6th",
    "calctapp_app_div",
    "calctapp_u",
    "colo",
    "coloctapp",
    "conn",
    "connappct",
    "dc",
    "delaware",
    "delch",
    "delctcompl",
    "delsuperct",
    "fla",
    "fladistctapp_1_per_curiam",
    "fladistctapp_1_written",
    "fladistctapp_2_per_curiam",
    "fladistctapp_2_written",
    "fladistctapp_3",
    "fladistctapp_4_pc",
    "fladistctapp_4_written",
    "fladistctapp_5",
    "ga",
    "gactapp",
    "haw",
    "haw_beginningofyear",
    "hawapp",
    "hawapp_beginningofyear",
    "idaho_civil",
    "idaho_criminal",
    "idahoctapp_civil",
    "idahoctapp_criminal",
    "idahoctapp_u",
    "ill",
    "illappct",
    "ind",
    "indctapp",
    "indtc",
    "iowa",
    "iowactapp",
    "kan_p",
    "kan_u",
    "kanctapp_p",
    "kanctapp_u",
    "ky",
    "la",
    "mass",
    "massappct",
    "md",
    "mdag",
    "mdctspecapp",
    "me",
    "mich",
    "michctapp_p",
    "michctapp_u",
    "minn",
    "minnag",
    "minnctapp",
    "miss",
    "missctapp",
    "mo",
    "moctapp_eastern",
    "moctapp_southern",
    "moctapp_western",
    "mont",
    "nc",
    "ncctapp",
    "neb",
    "nebctapp",
    "nd",
    "nev_p",
    "nev_u",
    "nh",
    "nm",
    "nmctapp",
    "nj",
    "njsuperctappdiv",
    "ny",
    "nyag",
    "nyappdiv_1st",
    "nyappdiv_2nd",
    "nyappdiv_3rd",
    "nyappdiv_4th",
    "nyappterm_1st",
    "nyappterm_2nd",
    "nysupct",
    "ohio",
    "ohioctapp_1",
    "ohioctapp_2",
    "ohioctapp_3",
    "ohioctapp_4",
    "ohioctapp_5",
    "ohioctapp_6",
    "ohioctapp_7",
    "ohioctapp_8",
    "ohioctapp_9",
    "ohioctapp_10",
    "ohioctapp_11",
    "ohioctapp_12",
    "ohioctcl",
    "ohioctcl_beginningofyear",
    "or",
    "okla",
    "oklaag",
    "oklacivapp",
    "oklacrimapp",
    "pa",
    "pacommwct",
    "pasuperct",
    "ri_p",
    "ri_u",
    "sd",
    "sc",
    "scctapp",
    "tenn",
    "tennctapp",
    "tenncrimapp",
    "tex",
    "texag",
    "texapp_1",
    "texapp_2",
    "texapp_3",
    "texapp_4",
    "texapp_5",
    "texapp_6",
    "texapp_7",
    "texapp_8",
    "texapp_9",
    "texapp_10",
    "texapp_11",
    "texapp_12",
    "texapp_13",
    "texapp_14",
    "texcrimapp",
    "utah",
    "va",
    "vactapp_u",
    "vactapp_p",
    "vt",
    "vtsuperct_civil",
    "vtsuperct_criminal",
    "vtsuperct_environmental",
    "vtsuperct_family",
    "vtsuperct_probate",
    "wash",
    "washctapp_p",
    "washctapp_u",
    "wis",
    "wva",
    "wyo",
]



"""Scraper for Federal Circuit of Appeals
CourtID: cafc
Court Short Name: cafc
Author: Andrei Chelaru
Reviewer: mlr
History:
 - created by Andrei Chelaru, 18 July 2014
 - Updated/rewritten by mlr, 2016-04-14
"""

from datetime import date
from dateutil.rrule import DAILY, rrule
from juriscraper.OralArgumentSite import OralArgumentSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        d = date.today()
        self.url = "http://www.cafc.uscourts.gov/oral-argument-recordings?field_date_value2[value][date]={date}".format(
            date=d.strftime("%Y-%m-%d")
        )
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                DAILY,
                interval=1,  # Every day
                dtstart=date(2015, 7, 10),
                until=date(2016, 4, 14),
            )
        ]

    def _get_download_urls(self):
        path = "//td[contains(@class,'views-field-field-filename')]//@href"
        return list(self.html.xpath(path))

    def _get_case_names(self):
        path = "//td[contains(@class,'views-field-title')]//text()"
        return [" ".join(s.split()) for s in self.html.xpath(path)]

    def _get_case_dates(self):
        path = "//span[@class='date-display-single']/@content"
        return [convert_date_string(s.strip()) for s in self.html.xpath(path)]

    def _get_docket_numbers(self):
        path = "//td[contains(@class,'views-field-field-case-number')]//text()"
        return [s.strip() for s in self.html.xpath(path)]

    def _download_backwards(self, d):
        self.url = (
            self.url
        ) = "http://www.cafc.uscourts.gov/oral-argument-recordings?field_date_value2[value][date]={date}".format(
            date=d.strftime("%Y-%m-%d")
        )
        self.html = self._download()



# Scraper for the United States Court of Appeals for the Seventh Circuit
# CourtID: ca7
# Court Short Name: 7th Cir.

import time
from six.moves.urllib.parse import urlencode
from datetime import date, timedelta
from dateutil.rrule import rrule, DAILY
from lxml import html

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.a_while_ago = date.today() - timedelta(days=60)
        self.url = (
            "http://media.ca7.uscourts.gov/cgi-bin/rssExec.pl?Time=any&FromMonth={month}&FromDay={day}&FromYear={year}&"
            "ToMonth=&ToDay=&ToYear=&Author=any&AuthorName=&Case=any&CaseY1=&CaseY2=&CaseN1=&CaseN2=&CaseN3=&"
            "CaseN4=&Submit=Submit&RssJudgeName=Easterbrook&OpsOnly=no".format(
                month=self.a_while_ago.month,
                day=self.a_while_ago.day,
                year=self.a_while_ago.year,
            )
        )
        self.court_id = self.__module__

        self.interval = 30
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                DAILY,
                interval=self.interval,  # Every interval days
                dtstart=date(1999, 10, 1),
                until=date(2015, 1, 1),
            )
        ]

    def _get_case_names(self):
        case_names = []
        for e in self.html.xpath("//table//table/tr[position() >= 3]/td[2]"):
            s = html.tostring(e, method="text", encoding="unicode")
            case_names.append(s)
        return case_names

    def _get_download_urls(self):
        return [
            e
            for e in self.html.xpath(
                "//table[2]/tr/td/table/tr[position() >=3]/td/a/@href"
            )
        ]

    def _get_case_dates(self):
        return [
            date.fromtimestamp(
                time.mktime(time.strptime(date_string.strip(), "%m/%d/%Y"))
            )
            for date_string in self.html.xpath(
                "//table//table/tr[position() >= 3]/td[4]/text()"
            )
        ]

    def _get_docket_numbers(self):
        return [
            docket_number
            for docket_number in self.html.xpath(
                "//table//table/tr[position() >= 3]/td[1]/text()"
            )
        ]

    def _get_precedential_statuses(self):
        statuses = []
        for e in self.html.xpath("//table//table/tr[position() >= 3]/td[5]/a"):
            s = html.tostring(e, method="text", encoding="unicode")
            if "Opinion" in s:
                statuses.append("Published")
            elif "Nonprecedential" in s:
                statuses.append("Unpublished")
            else:
                statuses.append("Unknown")
        return statuses

    def _get_nature_of_suit(self):
        natures = []
        for e in self.html.xpath("//table//table/tr[position() >= 3]/td[3]"):
            natures.append(html.tostring(e, method="text", encoding="unicode"))
        return natures

    def _get_judges(self):
        judges = []
        for e in self.html.xpath("//table//table/tr[position() >= 3]/td[6]"):
            s = html.tostring(e, method="text", encoding="unicode")
            if s.lower().strip() == "percuriam":
                s = "Per Curiam"
            judges.append(s)
        return judges

    def _download_backwards(self, d):
        to_date = d + timedelta(self.interval)
        params = urlencode(
            {
                "Time": "any",
                "FromMonth": d.month,
                "FromDay": d.day,
                "FromYear": d.year,
                "ToMonth": to_date.month,
                "ToDay": to_date.day,
                "ToYear": to_date.year,
                "Author": "any",
                "AuthorName": "",
                "Case": "any",
                "CaseY1": "",
                "CaseY2": "",
                "CaseN1": "",
                "CaseN2": "",
                "CaseN3": "",
                "CaseN4": "",
                "Submit": "Submit",
                "RssJudgeName": "Easterbrook",
                "OpsOnly": "no",
            }
        )
        self.base_url = "http://media.ca7.uscourts.gov/cgi-bin/rssExec.pl"
        self.url = "{}?{}".format(self.base_url, params)

        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200



from lxml import html
from datetime import datetime, timedelta, date
from dateutil.rrule import DAILY, rrule
from selenium.common.exceptions import NoSuchElementException
from juriscraper.AbstractSite import logger
from juriscraper.OpinionSiteWebDriven import OpinionSiteWebDriven


class Site(OpinionSiteWebDriven):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = "http://www.ca5.uscourts.gov/electronic-case-filing/case-information/current-opinions"
        self.court_id = self.__module__
        self.interval = 5
        self.case_date = datetime.today()
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                DAILY,
                interval=self.interval,  # Every interval days
                dtstart=date(1992, 5, 14),
                until=date(2015, 1, 1),
            )
        ]
        self.uses_selenium = True

    def _download(self, request_dict={}):
        if self.test_mode_enabled():
            html_tree_list = [
                super(Site, self)._download(request_dict=request_dict)
            ]
            self.records_nr = len(
                html_tree_list[0].xpath(
                    "//tr[contains(concat('', @id, ''), 'ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00')]"
                )
            )
            return html_tree_list
        else:
            logger.info("Running Selenium browser...")
            self.initiate_webdriven_session()
            self.wait_for_id("ctl00_Body_C010_ctl00_ctl00_endDate_dateInput")

            start_date = self.webdriver.find_element_by_id(
                "ctl00_Body_C010_ctl00_ctl00_startDate_dateInput"
            )
            start_date.send_keys(
                (self.case_date - timedelta(days=self.interval)).strftime(
                    "%m/%d/%Y"
                )
            )

            end_date = self.webdriver.find_element_by_id(
                "ctl00_Body_C010_ctl00_ctl00_endDate_dateInput"
            )
            end_date.send_keys(self.case_date.strftime("%m/%d/%Y"))

            submit = self.webdriver.find_element_by_id(
                "Body_C010_ctl00_ctl00_btnSearch"
            )
            submit.click()

            self.wait_for_id(
                "ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00"
            )

            self.status = 200

            try:
                nr_of_pages = self.webdriver.find_element_by_xpath(
                    '//div[contains(concat(" ", @class, " "), "rgInfoPart")]/strong[2]'
                )
                records_nr = self.webdriver.find_element_by_xpath(
                    '//div[contains(concat(" ", @class, " "), "rgInfoPart")]/strong[1]'
                )
                self.records_nr = int(records_nr.text)
                nr_of_pages = int(nr_of_pages.text)
            except NoSuchElementException:
                try:
                    self.records_nr = len(
                        self.webdriver.find_elements_by_xpath(
                            "//tr[contains(concat('', @id, ''), 'ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00')]"
                        )
                    )
                    nr_of_pages = 1
                except NoSuchElementException:
                    self.webdriver.quit()
                    return []
            html_pages = []
            logger.info(
                "records: {}, pages: {}".format(self.records_nr, nr_of_pages)
            )
            if nr_of_pages == 1:
                text = self.webdriver.page_source
                self.webdriver.quit()

                html_tree = html.fromstring(text)
                html_tree.make_links_absolute(self.url)

                remove_anchors = lambda url: url.split("#")[0]
                html_tree.rewrite_links(remove_anchors)
                html_pages.append(html_tree)
            else:
                logger.info(
                    "Paginating through %s pages of results." % nr_of_pages
                )
                logger.info("  Getting page 1")
                text = self.webdriver.page_source

                html_tree = html.fromstring(text)
                html_tree.make_links_absolute(self.url)

                remove_anchors = lambda url: url.split("#")[0]
                html_tree.rewrite_links(remove_anchors)
                html_pages.append(html_tree)

                for i in range(nr_of_pages - 1):
                    logger.info("  Getting page %s" % (i + 2))
                    next_page = self.webdriver.find_element_by_class_name(
                        "rgPageNext"
                    )
                    next_page.click()
                    self.webdriver.implicitly_wait(5)

                    text = self.webdriver.page_source

                    html_tree = html.fromstring(text)
                    html_tree.make_links_absolute(self.url)

                    remove_anchors = lambda url: url.split("#")[0]
                    html_tree.rewrite_links(remove_anchors)
                    html_pages.append(html_tree)
                self.webdriver.quit()
            return html_pages

    def _get_case_names(self):
        case_names = []
        for html_tree in self.html:
            page_records_count = self._get_opinion_count(html_tree)
            for record in range(page_records_count):
                path = "id('ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00__{n}')/td[4]/text()".format(
                    n=record
                )
                case_names.append(html_tree.xpath(path)[0])
        return case_names

    def _get_download_urls(self):
        for html_tree in self.html:
            page_records_count = self._get_opinion_count(html_tree)
            for record in range(page_records_count):
                path = "id('ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00__{n}')/td[2]/a/@href".format(
                    n=record
                )
                yield html_tree.xpath(path)[0]

    def _get_case_dates(self):
        for html_tree in self.html:
            page_records_count = self._get_opinion_count(html_tree)
            for record in range(page_records_count):
                path = "id('ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00__{n}')/td[3]/text()".format(
                    n=record
                )
                yield datetime.strptime(html_tree.xpath(path)[0], "%m/%d/%Y")

    def _get_docket_numbers(self):
        for html_tree in self.html:
            page_records_count = self._get_opinion_count(html_tree)
            for record in range(page_records_count):
                path = "id('ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00__{n}')/td[2]/a/text()".format(
                    n=record
                )
                yield html_tree.xpath(path)[0]

    def _get_precedential_statuses(self):
        for html_tree in self.html:
            page_records_count = self._get_opinion_count(html_tree)
            for record in range(page_records_count):
                path = "id('ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00__{n}')/td[5]/text()".format(
                    n=record
                )
                yield "Unpublished" if "unpub" in html_tree.xpath(path)[
                    0
                ] else "Published"

    @staticmethod
    def _get_opinion_count(html_tree):
        return int(
            html_tree.xpath(
                "count(//tr[contains(concat('', @id, ''), 'ctl00_Body_C010_ctl00_ctl00_radGridOpinions_ctl00')])"
            )
        )

    def _download_backwards(self, d):
        self.case_date = d
        logger.info(
            "Running backscraper with date range: %s to %s"
            % (self.case_date - timedelta(days=self.interval), self.case_date,)
        )
        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200



from juriscraper.OpinionSite import OpinionSite
import re
import time
from datetime import date
from dateutil.rrule import MONTHLY, rrule


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        today = date.today()
        self.url = (
            "http://media.ca8.uscourts.gov/cgi-bin/opnByMM.pl?theMM=%02d&theYY=%s&A1=Get+Opinions"
            % (today.month, today.year)
        )
        self.court_id = self.__module__

        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                MONTHLY, dtstart=date(1995, 11, 1), until=date(2015, 1, 1),
            )
        ]

    def _get_case_names(self):
        case_names = []
        case_name_regex = re.compile("(\d{2}/\d{2}/\d{4})(.*)")
        for text in self.html.xpath(
            '//a[contains(@href, "opndir")]/following-sibling::b/text()'
        ):
            case_names.append(case_name_regex.search(text).group(2))
        return case_names

    def _get_download_urls(self):
        return [
            e for e in self.html.xpath('//a[contains(@href, "opndir")]/@href')
        ]

    def _get_case_dates(self):
        case_dates = []
        case_date_regex = re.compile("(\d{2}/\d{2}/\d{4})(.*)")
        for text in self.html.xpath(
            '//a[contains(@href, "opndir")]/following-sibling::b/text()'
        ):
            date_string = case_date_regex.search(text).group(1)
            case_dates.append(
                date.fromtimestamp(
                    time.mktime(time.strptime(date_string, "%m/%d/%Y"))
                )
            )
        return case_dates

    def _get_docket_numbers(self):
        docket_numbers = []
        docket_number_regex = re.compile("(\d{2})(\d{4})(u|p)", re.IGNORECASE)
        for docket_number in self.html.xpath(
            '//a[contains(@href, "opndir")]/text()'
        ):
            regex_results = docket_number_regex.search(docket_number)
            docket_numbers.append(
                "%s-%s" % (regex_results.group(1), regex_results.group(2))
            )
        return docket_numbers

    def _get_precedential_statuses(self):
        statuses = []
        for docket_number in self.html.xpath(
            '//a[contains(@href, "opndir")]/text()'
        ):
            docket_number = docket_number.split(".")[0]
            if "p" in docket_number.lower():
                statuses.append("Published")
            elif "u" in docket_number.lower():
                statuses.append("Unpublished")
            else:
                statuses.append("Unknown")
        return statuses

    def _download_backwards(self, d):

        self.url = (
            "http://media.ca8.uscourts.gov/cgi-bin/opnByMM.pl?theMM=%02d&theYY=%s&A1=Get+Opinions"
            % (d.month, d.year)
        )

        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200



from juriscraper.opinions.united_states.state import cal


class Site(cal.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.url = (
            "http://www.courtinfo.ca.gov/cgi-bin/opinarch-blank.cgi?Courts=S"
        )
        self.court_id = self.__module__



"""Scraper for South Carolina Supreme Court
CourtID: sc
Court Short Name: S.C.
Author: TBM <-- Who art thou TBM? ONLY MLR gets to be initials!
History:
 - 04-18-2014: Created.
 - 09-18-2014: Updated by mlr.
 - 10-17-2014: Updated by mlr to fix InsanityError
 - 2017-10-04: Updated by mlr to deal with their anti-bot system. Crux of change
               is to ensure that we get a cookie in our session by visiting the
               homepage before we go and scrape. Dumb.
 - 2019-02-26: Restructured completely by arderyp

Contact information:
 - Help desk: (803) 734-1193, Travis
 - Security desk: 803-734-5906, Joe Hilke
 - Web Developer (who can help with firewall issues): 803-734-0373, Winkie Clark
"""

import datetime

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.exceptions import InsanityException
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        today = datetime.date.today()
        self.url = (
            "http://www.sccourts.org/opinions/indexSCPub.cfm?year=%d&month=%d"
            % (today.year, today.month)
        )
        self.cases = []

    def _process_html(self):
        path = '//a[@class="blueLink2"][contains(./@href, ".pdf") or contains(./@href, "?orderNo=")]'
        for link in self.html.xpath(path):
            docket, name, url = self.extract_docket_name_url_from_link(link)
            date, summary = self.extract_date_summary_from_link(link)
            self.cases.append(
                {
                    "date": date,
                    "docket": docket,
                    "name": name,
                    "url": url,
                    "summary": summary,
                }
            )

    def extract_docket_name_url_from_link(self, link):
        url = link.attrib["href"]
        text = link.text_content().strip()
        parts = text.split("-", 1)
        name = parts[1]
        docket = parts[0].strip()
        docket = "" if "order" in docket.lower() else docket
        return docket, name, url

    def extract_date_summary_from_link(self, link):
        # Link should be within a <p> tag directly under <div id='maincontent'>, but
        # occasionally the courts forgets the wrap it in a <p>, in which case it should
        # be directly under the <div id='maincontent'>
        container_id = "maincontent"
        parent = link.getparent()
        parents_parent = parent.getparent()
        if "id" in parent.attrib and parent.attrib["id"] == container_id:
            search_root = link
        elif (
            "id" in parents_parent.attrib
            and parents_parent.attrib["id"] == container_id
        ):
            search_root = parent
        else:
            raise InsanityException(
                'Unrecognized placement of Opinion url on page: "%s"'
                % link.text_content().strip()
            )

        # Find date from bolded header element above link (ex: "5-14-2014 - Opinions" or "5-21-2014 - Orders")
        element_date = search_root.xpath("./preceding-sibling::b")[-1]
        element_date_text = element_date.text_content().strip().lower()
        try:
            date_string = element_date_text.split()[0]
        except:
            raise InsanityException(
                'Unrecognized bold (date) element: "%s"' % element_date_text
            )

        # Find summary from blockquote element below link
        element_blockquote = search_root.xpath(
            "./following-sibling::blockquote"
        )[0]
        summary = element_blockquote.text_content().strip()

        return convert_date_string(date_string), summary

    def _get_download_urls(self):
        return [case["url"] for case in self.cases]

    def _get_case_names(self):
        return [case["name"] for case in self.cases]

    def _get_case_dates(self):
        return [case["date"] for case in self.cases]

    def _get_docket_numbers(self):
        return [case["docket"] for case in self.cases]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.cases)

    def _get_summaries(self):
        return [case["summary"] for case in self.cases]



"""Scraper for the Rhode Island Supreme Court
CourtID: ri
Court Short Name: R.I.
Court Contact: (see ri_p)
Author: Brian W. Carver
Date created: 2013-08-10
"""

from juriscraper.opinions.united_states.state import ri_p


class Site(ri_p.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.include_summary = False
        self.status = "Unpublished"
        self.url = self.build_url(
            "https://www.courts.ri.gov/Courts/SupremeCourt/Pages/Orders/Orders"
        )

    def _get_summaries(self):
        # No summaries for unpublished, just short-circuit.
        return None



# Scraper for Oklahoma Court of Criminal Appeals
# CourtID: oklacrimapp
# Court Short Name: OK
# Author: Andrei Chelaru
# Reviewer: mlr
# Date: 2014-07-05

from datetime import date

from juriscraper.opinions.united_states.state import okla


class Site(okla.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        d = date.today()
        self.url = "http://www.oscn.net/applications/oscn/Index.asp?ftdb=STOKCSCR&year={year}&level=1".format(
            year=d.year
        )



# # coding=utf-8

from juriscraper.OpinionSiteLinear import OpinionSiteLinear
from juriscraper.lib.exceptions import InsanityException
from juriscraper.lib.string_utils import clean_string
from juriscraper.lib.string_utils import convert_date_string


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.isc.idaho.gov/appeals-court/coaunpublished"
        self.status = "Unpublished"

    def _process_html(self):
        for item in self.html.xpath('//li[contains(.//a/@href, ".pdf")]'):
            text = clean_string(item.text_content())
            date_string = " ".join(text.split()[0:3])
            try:
                convert_date_string(date_string)
            except:
                raise InsanityException('Unexpected text format: "%s"' % text)
            docket_name = text.replace(date_string, "").strip().lstrip("-")

            # sometimes the records include a docket number(s) as the
            # first words in the second half of the hyphenated string,
            # but some don't include a docket at all.  So we test to see
            # if the first word is numeric (minus the slash characters
            # used to conjoin multiple docket numbers).
            docket, name = docket_name.split(None, 1)
            first_word = docket[0].replace("/", "")
            if not first_word.isnumeric():
                docket = ""
                name = docket_name

            self.cases.append(
                {
                    "date": date_string,
                    "docket": docket,
                    "name": name,
                    "url": item.xpath(".//a/@href")[0],
                }
            )



# Scraper for Oklahoma Attorney General Opinions
# CourtID: oklaag
# Court Short Name: OK
# Author: Andrei Chelaru
# Reviewer: mlr
# Date: 2014-07-05

from datetime import date

from juriscraper.opinions.united_states.state import okla
from lxml import html

## WARNING: THIS SCRAPER IS FAILING:
## This scraper is succeeding in development, but
## is failing in production.  We are not exactly
## sure why, and suspect that the hosting court
## site may be blocking our production IP and/or
## throttling/manipulating requests from production.


class Site(okla.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        d = date.today()
        self.url = "http://www.oscn.net/applications/oscn/Index.asp?ftdb=STOKAG&year={year}&level=1".format(
            year=d.year
        )
        self.court_id = self.__module__

    def _get_precedential_statuses(self):
        return ["Unpublished"] * len(self.case_names)

    @staticmethod
    def cleanup_content(content):
        tree = html.fromstring(content)
        core_element = tree.xpath(
            '//div[contains(concat(" ", normalize-space(@class), " "), " main ")]'
        )[0]
        return html.tostring(
            core_element, pretty_print=True, encoding="unicode"
        )



# Scraper for the United States Court of International Trade
# CourtID: cit
# Court Short Name: Ct. Int'l Trade
# Neutral Citation Format: Ct. Int'l Trade No. 12-1
from juriscraper.OpinionSite import OpinionSite
import re
import time
from datetime import date
from lxml import html


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        # This is a special backscraper to deal with problems on the 2005 page.
        self.url = "http://www.cit.uscourts.gov/SlipOpinions/SlipOps-2005.html"
        self.court_id = self.__module__

    def _get_download_urls(self):
        return [
            t
            for t in self.html.xpath(
                "//table[3]/tr[position() > 1]/td[1]/a/@href | //table[3]/tr[position() > 1]/td[1]/font//@href"
            )
        ]

    def _get_neutral_citations(self):
        neutral_citations = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[1]/font/a"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            neutral_citations.append(s)
        return neutral_citations

    def _get_case_names(self):
        case_names = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[2]/font"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            # We strip "erratum: mm/dd/yyyy" from the case names of errata docs.
            if "erratum" in s:
                case_names.append(s.strip()[:-18])
            else:
                case_names.append(s.strip())
        return case_names

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_case_dates(self):
        case_dates = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[3]/font"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            case_dates.append(
                date.fromtimestamp(
                    time.mktime(time.strptime(s.strip(), "%m/%d/%Y"))
                )
            )
        return case_dates

    # Because there can be multiple docket numbers we have to replace some newlines.
    def _get_docket_numbers(self):
        docket_numbers = []
        for e in self.html.xpath("//table[3]/tr[position() > 1]/td[4]"):
            s = html.tostring(e, method="text", encoding="unicode").strip()
            docket_numbers.append(s.replace("\r\n", " &"))
        return docket_numbers
